import math
import matplotlib as mpl
from annogesiclib.gff3 import Gff3Parser
from annogesiclib.helper import Helper
import numpy as np
mpl.use('Agg')
import matplotlib.pyplot as plt
plt.style.use('ggplot')


def plot(utr, utr_pri, utr_sec, filename, source, utr_type, base_5utr):
    '''plot the distribution of length of UTRs'''
    bin_num = np.arange(0, 300, 5)
    if utr_type == "5utr":
        if source and (base_5utr != "transcript"):
            n, bins, hist1 = plt.hist(utr, bin_num,
                                      color="#FF9999", label='secondary')
            n, bins, hist2 = plt.hist(utr_pri, bin_num,
                                      color="#9999FF", label='primary')
            plt.legend((hist2[0], hist1[0]),
                       ("Primary TSSs", "Secondary TSSs"))
        else:
            n, bins, hist1 = plt.hist(utr, bin_num, color="#FF9999")
        plt.xlabel("5'UTR_length")
    elif utr_type == "3utr":
        n, bins, hist = plt.hist(utr, bin_num, color="#9999FF", label='3\'UTR')
        plt.legend([hist[0]], ["3'UTR"])
        plt.xlabel("3'UTR_length")
    plt.ylabel("Amount")
    plt.savefig(filename)
    plt.clf()


def get_feature(cds):
    if "protein_id" in cds.attributes.keys():
        cds_name = cds.attributes["protein_id"]
    elif "locus_tag" in cds.attributes.keys():
        cds_name = cds.attributes["locus_tag"]
    else:
        strand = Helper().get_strand_name(cds.strand)
        cds_name = "".join([cds.feature, ":", str(cds.start),
                            "-", str(cds.end), "_", strand])
    return cds_name


def check_ta(tas, utr_start, utr_end, seq_id, strand):
    '''chekc transcript to verify the UTR'''
    detect = False
    for ta in tas:
        if (ta.seq_id == seq_id) and \
           (ta.strand == strand):
            if (ta.start <= utr_start) and \
               (ta.end >= utr_end):
                detect = True
                break
    return detect, ta


def import_utr(tss, utr_strain, utr_all, start, end, tas, length, args_utr):
    if args_utr.source:
        if "Primary" in tss.attributes["type"]:
            if args_utr.base_5utr.lower() == "both":
                detect, ta = check_ta(tas, start, end, tss.seq_id, tss.strand)
            elif args_utr.base_5utr.lower() == "tss":
                detect = True
            if detect:
                utr_strain["pri"][tss.seq_id].append(length)
                utr_all["pri"].append(length)
        elif "Secondary" in tss.attributes["type"]:
            if args_utr.base_5utr.lower() == "both":
                detect, ta = check_ta(tas, start, end, tss.seq_id, tss.strand)
            elif args_utr.base_5utr.lower() == "tss":
                detect = True
            if detect:
                utr_strain["sec"][tss.seq_id].append(length)
                utr_all["sec"].append(length)
    else:
        if args_utr.base_5utr.lower() == "both":
            detect, ta = check_ta(tas, start, end, tss.seq_id, tss.strand)
        elif args_utr.base_5utr.lower() == "tss":
            detect = True
    if detect:
        utr_strain["all"][tss.seq_id].append(length)
        utr_all["all"].append(length)
    if args_utr.base_5utr.lower() == "tss":
        ta = None
    return detect, ta


def get_print_string_5utr(num_utr, name_utr, length, tss, cds_name,
                          locus_tag, ta, source, out, start, end):
    attribute_string = ";".join(
             ["=".join(items) for items in [
              ("ID", "_".join([ta.seq_id, "utr5", str(num_utr)])),
              ("Name", "_".join(["5'UTR", name_utr])),
              ("length", str(length)),
              ("associated_cds", cds_name),
              ("associated_gene", locus_tag),
              ("associated_tss", tss.attributes["Name"])]])
    if source:
        attribute_string = ";".join([
            attribute_string, "=".join(["tss_type", tss.attributes["type"]])])
    if ta is not None:
        if "ID" in ta.attributes.keys():
            attribute_string = ";".join([attribute_string, "Parent=" + ta.attributes["ID"]])
        else:
            attribute_string = ";".join([
                attribute_string, "=".join(["Parent", "Transcript:" +
                                            str(ta.start) + "-" + str(ta.end) +
                                            "_" + ta.strand])])
    out.write("{0}\tANNOgesic\t5UTR\t{1}\t{2}\t.\t{3}\t.\t{4}\n".format(
              tss.seq_id, start, end,
              tss.strand, attribute_string))


def get_5utr(tss, near_cds, utr_strain, utr_all, tas, num_utr,
             cds_name, locus_tag, out, args_utr):
    '''print and import the 5UTR information'''
    if tss.strand == "+":
        start = tss.start
        end = near_cds.start
        length = end - start
        detect, ta = import_utr(tss, utr_strain, utr_all,
                                start, end, tas, length, args_utr)
    else:
        start = near_cds.end
        end = tss.end
        length = end - start
        detect, ta = import_utr(tss, utr_strain, utr_all,
                                start, end, tas, length, args_utr)
    if detect:
        name_utr = '%0*d' % (5, num_utr)
        get_print_string_5utr(num_utr, name_utr, length, tss, cds_name,
                              locus_tag, ta, args_utr.source, out, start, end)
        num_utr += 1
    return num_utr


def detect_cds(cdss, gene):
    '''get the information of CDS'''
    detect = False
    for cds in cdss:
        if "Parent" in cds.attributes.keys():
            if gene.attributes["ID"] in cds.attributes["Parent"].split(","):
                detect = True
                near_cds = cds
                check_utr = True
                cds_name = get_feature(cds)
        else:
            if "locus_tag" in cds.attributes.keys():
                if gene.attributes["locus_tag"] == cds.attributes["locus_tag"]:
                    near_cds = cds
                    cds_name = cds.attributes["locus_tag"]
                    check_utr = True
                    detect = True
            elif (gene.seq_id == cds.seq_id) and \
                 (gene.strand == cds.strand):
                if (gene.start >= cds.start) and \
                   (gene.end <= cds.end):
                    near_cds = cds
                    cds_name = get_feature(cds)
                    check_utr = True
                    detect = True
    if not detect:
        return None, None, False
    else:
        return near_cds, cds_name, check_utr


def read_file(tss_file, gff_file, ta_file, term_file):
    genes = []
    cdss = []
    terms = []
    tsss = []
    tas = []
    gff_f = open(gff_file, "r")
    for entry in Gff3Parser().entries(gff_f):
        if (entry.feature == "CDS"):
            cdss.append(entry)
        elif (entry.feature == "gene"):
            genes.append(entry)
    if ta_file is not None:
        for entry in Gff3Parser().entries(open(ta_file)):
            tas.append(entry)
        tas = sorted(tas, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    if term_file is not None:
        for entry_term in Gff3Parser().entries(open(term_file)):
            terms.append(entry_term)
        terms = sorted(terms, key=lambda k: (k.seq_id, k.start,
                                             k.end, k.strand))
    if tss_file is not None:
        for entry in Gff3Parser().entries(open(tss_file)):
            tsss.append(entry)
        tsss = sorted(tsss, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    genes = sorted(genes, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    cdss = sorted(cdss, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    return genes, cdss, terms, tsss, tas


def check_associated_TSSpredator(genes, tss, cdss, check_utr, cds_name, locus):
    '''get the associated TSS which is generated from TSSpredator'''
    near_cds = None
    for gene in genes:
        if (tss.seq_id == gene.seq_id) and (
                tss.strand == gene.strand):
            if "locus_tag" in gene.attributes.keys():
                if gene.attributes["locus_tag"] == locus:
                    for cds in cdss:
                        if "Parent" in cds.attributes.keys():
                            if (gene.attributes["ID"] in
                                    cds.attributes["Parent"].split(",")):
                                near_cds = cds
                                check_utr = True
                                cds_name = get_feature(cds)
                        else:
                            if "locus_tag" in cds.attributes.keys():
                                if (gene.attributes["locus_tag"] ==
                                        cds.attributes["locus_tag"]):
                                    near_cds = cds
                                    cds_name = cds.attributes["locus_tag"]
                                    check_utr = True
                            elif (gene.seq_id == cds.seq_id) and (
                                    gene.strand == cds.strand):
                                if (gene.start >= cds.start) and (
                                        gene.end <= cds.end):
                                    near_cds = cds
                                    cds_name = get_feature(cds)
                                    check_utr = True
    return check_utr, cds_name, near_cds


def get_5utr_from_TSSpredator(tss, genes, cdss):
    '''It is for TSS file which is generated from ANNOgesic'''
    check_utr = False
    cds_name = "NA"
    if ("Primary" in tss.attributes["type"]) or (
            "Secondary" in tss.attributes["type"]):
        ass_gene = tss.attributes["associated_gene"].split(",")
        tss_type = tss.attributes["type"].split(",")
        for index in range(len(tss_type)):
            if (tss_type[index] == "Primary") or (
                    tss_type[index] == "Secondary"):
                locus_tag = ass_gene[index]
                check_utr, cds_name, near_cds = check_associated_TSSpredator(
                                                genes, tss, cdss, check_utr,
                                                cds_name, locus_tag)
                if not check_utr:
                    for cds in cdss:
                        if (cds.seq_id == tss.seq_id) and (
                                cds.strand == tss.strand):
                            strand = Helper().get_strand_name(cds.strand)
                            if locus_tag == (cds.feature + ":" +
                                             str(cds.start) +
                                             "-" + str(cds.end) +
                                             "_" + strand):
                                near_cds = cds
                                cds_name = get_feature(cds)
                                check_utr = True
        if check_utr:
            utr_datas = {"check": check_utr, "cds_name": cds_name,
                         "near_cds": near_cds, "locus": locus_tag}
        else:
            utr_datas = {"check": False, "cds_name": None,
                         "near_cds": None, "locus": None}
    else:
        utr_datas = {"check": False, "cds_name": None,
                     "near_cds": None, "locus": None}
    return utr_datas


def get_5utr_from_other(tss, genes, cdss, length):
    '''It is for TSS file which is not generated from ANNOgesic'''
    check_utr = False
    cds_name = "NA"
    for gene in genes:
        if (tss.seq_id == gene.seq_id) and (
                tss.strand == gene.strand):
            if (tss.strand == "+") and (
                    (gene.start - tss.start) <= length) and (
                    tss.start <= gene.start):
                if "locus_tag" in gene.attributes.keys():
                    locus_tag = gene.attributes["locus_tag"]
                else:
                    locus_tag = gene.attributes["ID"]
                near_cds, cds_name, check_utr = detect_cds(cdss, gene)
                break
            elif (tss.strand == "-") and (
                    (tss.start - gene.end) <= length) and (
                    tss.start >= gene.end):
                if "locus_tag" in gene.attributes.keys():
                    locus_tag = gene.attributes["locus_tag"]
                else:
                    locus_tag = gene.attributes["ID"]
                near_cds, cds_name, check_utr = detect_cds(cdss, gene)
                break
    if check_utr:
        utr_datas = {"check": check_utr, "cds_name": cds_name,
                     "near_cds": near_cds, "locus": locus_tag}
    else:
        utr_datas = {"check": False, "cds_name": None,
                     "near_cds": None, "locus": None}
    return utr_datas


def get_attribute_string(num_utr, length, cds, gene_name, ta, id_name, name,
                         feature, feature_name):
    name_utr = '%0*d' % (5, num_utr)
    cds_name = get_feature(cds)
    attribute_string = ";".join(
                 ["=".join(items) for items in [
                  ("ID", "_".join([ta.seq_id, id_name, str(num_utr)])),
                  ("Name", "_".join([name, name_utr])),
                  ("length", str(length)),
                  ("associated_cds", cds_name),
                  ("associated_gene", gene_name)]])
    if feature == "Parent":
        if "ID" in ta.attributes.keys():
            attribute_string = ";".join([attribute_string,
                                         "Parent=" + ta.attributes["ID"]])
        else:
            attribute_string = ";".join(
                [attribute_string,
                 "Parent=" + (feature, feature_name + str(ta.start) + "-" +
                 str(ta.end) + "_" + ta.strand)])
    return attribute_string


def get_gene_name(genes, cds):
    for gene in genes:
        if (cds.seq_id == gene.seq_id) and (
                cds.strand == gene.strand):
            if ("Parent" in cds.attributes.keys()) and (
                    "ID" in gene.attributes.keys()):
                if gene.attributes["ID"] in cds.attributes["Parent"].split(","):
                    if "locus_tag" in gene.attributes.keys():
                        gene_name = gene.attributes["locus_tag"]
                    else:
                        gene_name = "Gene:" + str(gene.start) + "-" + \
                                    str(gene.end) + "_" + gene.strand
                    break
            if ((cds.start >= gene.start) and (
                    cds.end <= gene.end)):
                if "locus_tag" in gene.attributes.keys():
                    gene_name = gene.attributes["locus_tag"]
                else:
                    gene_name = ("Gene:" + str(gene.start) + "-" +
                                 str(gene.end) + "_" + gene.strand)
                break
    return gene_name


def set_utr_strain(ta, type_, utr_strain):
    if ta.seq_id not in utr_strain[type_].keys():
        utr_strain[type_][ta.seq_id] = []


def compare_ta(tas, genes, cdss, utr_strain, utr_all, out, args_utr):
    '''Comparing CDS and trancript to find the 5UTR'''
    num_utr = 0
    for ta in tas:
        detect = False
        set_utr_strain(ta, "all", utr_strain)
        set_utr_strain(ta, "pri", utr_strain)
        set_utr_strain(ta, "sec", utr_strain)
        for cds in cdss:
            if (ta.seq_id == cds.seq_id) and (
                    ta.strand == cds.strand):
                if ta.strand == "+":
                    if ((cds.start - ta.start) <= args_utr.length) and (
                            (cds.start - ta.start) >= 0):
                        if ((ta.start <= cds.start) and (
                                ta.end >= cds.end)) or (
                                (ta.start <= cds.start) and (
                                ta.end <= cds.end) and (
                                ta.end >= cds.start)):
                            length = cds.start - ta.start
                            utr_strain["all"][ta.seq_id].append(length)
                            utr_all["all"].append(length)
                            gene_name = get_gene_name(genes, cds)
                            string = get_attribute_string(
                                num_utr, length, cds, gene_name, ta, "utr5",
                                "5'UTR", "Parent", "Transcript:")
                            detect = True
                            start = ta.start
                            end = cds.start
                            break
                else:
                    if ((ta.end - cds.end) <= args_utr.length) and (
                            (ta.end - cds.end) >= 0):
                        if ((ta.start <= cds.start) and (
                                ta.end >= cds.end)) or (
                                (ta.start >= cds.start) and (
                                ta.start <= cds.end) and (
                                ta.end >= cds.end)):
                            near_cds = cds
                            detect = True
        if (ta.strand == "-") and (detect):
            length = ta.end - near_cds.end
            utr_strain["all"][ta.seq_id].append(length)
            utr_all["all"].append(length)
            gene_name = get_gene_name(genes, near_cds)
            string = get_attribute_string(
                num_utr, length, near_cds, gene_name,
                ta, "utr5", "5'UTR", "Parent", "Transcript:")
            start = near_cds.end
            end = ta.end
        if detect:
            if end - start > 0:
                out.write("{0}\tANNOgesic\t5UTR\t{1}"
                          "\t{2}\t.\t{3}\t.\t{4}\n".format(
                              ta.seq_id, start, end, ta.strand, string))
                num_utr += 1


def detect_5utr(tss_file, gff_file, ta_file, out_file, args_utr):
    '''detection of 5UTR'''
    num_utr = 0
    utr_all = {"all": [], "pri": [], "sec": []}
    utr_strain = {"all": {}, "pri": {}, "sec": {}}
    pre_seq_id = ""
    out = open(out_file, "w")
    out.write("##gff-version 3\n")
    genes, cdss, terms, tsss, tas = read_file(tss_file, gff_file,
                                              ta_file, None)
    if (args_utr.base_5utr.upper() == "TSS") or (
            args_utr.base_5utr.lower() == "both"):
        for tss in tsss:
            if args_utr.source:
                utr_datas = get_5utr_from_TSSpredator(tss, genes, cdss)
            else:
                utr_datas = get_5utr_from_other(tss, genes, cdss,
                                                args_utr.length)
            if utr_datas["check"]:
                if tss.seq_id != pre_seq_id:
                    pre_seq_id = tss.seq_id
                    utr_strain["pri"][tss.seq_id] = []
                    utr_strain["sec"][tss.seq_id] = []
                    utr_strain["all"][tss.seq_id] = []
                num_utr = get_5utr(tss, utr_datas["near_cds"], utr_strain,
                                   utr_all, tas, num_utr,
                                   utr_datas["cds_name"], utr_datas["locus"],
                                   out, args_utr)
    if args_utr.base_5utr.lower() == "transcript":
        compare_ta(tas, genes, cdss, utr_strain,
                   utr_all, out, args_utr)
    name = (gff_file.split("/"))[-1].replace(".gff", "")
    plot(utr_all["all"], utr_all["pri"], utr_all["sec"], "_".join([name,
         "all_5utr_length.png"]), args_utr.source, "5utr", args_utr.base_5utr)
    if len(utr_strain["all"]) > 1:
        for strain in utr_strain["all"].keys():
            plot(utr_strain["all"][strain], utr_strain["pri"][strain],
                 utr_strain["sec"][strain],
                 "_".join([strain, "5utr_length.png"]), args_utr.source,
                 "5utr", args_utr.base_5utr)
    out.close()


def compare_term(ta, terms, fuzzy):
    '''Comparing of transcript and terminator to get the 
    terminator which is associated with transcript'''
    for term in terms:
        if ta.strand == term.strand:
            if term.strand == "+":
                if (math.fabs(ta.end - term.start) <= fuzzy) or \
                   ((ta.end >= term.start) and (ta.end <= term.end)):
                    return term
            else:
                if (math.fabs(ta.start - term.end) <= fuzzy) or \
                   ((ta.start >= term.start) and (ta.start <= term.end)):
                    return term


def get_3utr(ta, near_cds, utr_all, utr_strain,
             attributes, num_utr, out, args_utr):
    '''print the 3UTR'''
    if ta.strand == "+":
        start = near_cds.end
        end = ta.end
        length = ta.end - near_cds.end
        utr_all.append(length)
        utr_strain[ta.seq_id].append(length)
    else:
        start = ta.start
        end = near_cds.start
        length = near_cds.start - ta.start
        utr_all.append(length)
        utr_strain[ta.seq_id].append(length)
    attributes.append("=".join(["length", str(length)]))
    if "ID" not in ta.attributes.keys():
        attributes.append("=".join([
            "Parent", "Transcript:" + str(ta.start) +
            "-" + str(ta.end) + "_" + ta.strand]))
    else:
        attributes.append("=".join([
            "Parent", ta.attributes["ID"]]))
    attribute = ";".join(attributes)
    if (length <= args_utr.length) and (length > 0):
        name_utr = '%0*d' % (5, num_utr)
        name = "=".join(["Name", "_".join(["3'UTR", name_utr])])
        id_ = "ID=" + ta.seq_id + "_utr3_" + str(num_utr)
        num_utr += 1
        attribute_string = ";".join([id_, name, attribute])
        if args_utr.base_3utr == "transcript":
            out.write("\t".join([ta.seq_id, "ANNOgesic", "3UTR",
                                 str(start), str(end),
                                 ta.score, ta.strand, ta.phase,
                                 attribute_string]) + "\n")
        elif args_utr.base_3utr == "both":
            if "associated_term=NA" not in attributes:
                out.write("\t".join([ta.seq_id, "ANNOgesic", "3UTR",
                                     str(start), str(end),
                                     ta.score, ta.strand, ta.phase,
                                     attribute_string]) + "\n")
    return num_utr


def get_near_cds(cdss, genes, ta, attributes):
    '''Get the associated CDS of terminator'''
    first = True
    detect = False
    for cds in cdss:
        if (ta.seq_id == cds.seq_id) and \
           (ta.strand == cds.strand):
            if first:
                near_cds = cds
                first = False
            if ta.strand == "+":
                if (cds.end <= ta.end) and \
                   (cds.start >= ta.start) and \
                   (cds.end > near_cds.end):
                    near_cds = cds
                    detect = True
            else:
                if (cds.start >= ta.start) and \
                   (cds.end <= ta.end):
                    near_cds = cds
                    detect = True
                    break
    if detect:
        for gene in genes:
            if ("Parent" in near_cds.attributes.keys()):
                if gene.attributes["ID"] in near_cds.attributes["Parent"].split(","):
                    attributes.append("=".join(["associated_gene",
                                      gene.attributes["locus_tag"]]))
        if "protein_id" in near_cds.attributes.keys():
            attributes.append("=".join(["associated_cds",
                              near_cds.attributes["protein_id"]]))
        elif "locus_tag" in near_cds.attributes.keys():
            attributes.append("=".join(["associated_cds",
                              near_cds.attributes["locus_tag"]]))
        else:
            attributes.append("=".join(["associated_cds",
                              "_".join([near_cds.feature,
                                        str(near_cds.start), str(near_cds.end),
                                        near_cds.strand])]))
    return near_cds


def compare_term_3utr(terms, cdss, genes, utr_all, utr_strain, args_utr, out):
    '''Comparing of terminator and 3UTR to get the relationship'''
    num_utr = 0
    for term in terms:
        detect = False
        if term.seq_id not in utr_strain.keys():
            utr_strain[term.seq_id] = []
        for cds in cdss:
            if (cds.seq_id == term.seq_id) and (
                    cds.strand == term.strand):
                if term.strand == "+":
                    if (term.end >= cds.end) and (
                            ((term.end - cds.end) <= args_utr.length) or (
                            ((term.start - cds.end) <= args_utr.length) and (
                            term.start >= cds.end))):
                        detect = True
                        near_cds = cds
                else:
                    if (term.start <= cds.start) and (
                            ((cds.start - term.start) <= args_utr.length) or (
                            ((cds.start - term.end) <= args_utr.length) and (
                            cds.start >= term.end))):
                        length = term.start - cds.start
                        utr_strain[term.seq_id].append(length)
                        utr_all.append(length)
                        gene_name = get_gene_name(genes, cds)
                        string = get_attribute_string(
                            num_utr, length, cds, gene_name, term, "utr3",
                            "3'UTR", "associated_term", "Terminator:")
                        detect = True
                        start = term.start
                        end = cds.start
                        break
            if (term.strand == "+") and detect:
                length = term.end - near_cds.end
                utr_strain[term.seq_id].append(length)
                utr_all.append(length)
                gene_name = get_gene_name(genes, near_cds)
                string = get_attribute_string(
                    num_utr, length, near_cds, gene_name, term, "utr3",
                    "3'UTR", "associated_term", "Terminator:")
                detect = True
                start = near_cds.end
                end = term.end
        if detect:
            if end - start > 0:
                out.write("{0}\tANNOgesic\t5UTR\t{1}\t"
                          "{2}\t.\t{3}\t.\t{4}\n".format(
                              term.seq_id, start, end, term.strand, string))
                num_utr += 1


def detect_3utr(ta_file, gff_file, term_file, out_file, args_utr):
    '''For detection of 3UTR'''
    num_utr = 0
    utr_all = []
    utr_strain = {}
    pre_seq_id = ""
    out = open(out_file, "w")
    out.write("##gff-version 3\n")
    genes, cdss, terms, tsss, tas = read_file(None, gff_file,
                                              ta_file, term_file)
    if (args_utr.base_3utr == "transcript") or (
            args_utr.base_3utr == "both"):
        for ta in tas:
            if term_file is not None:
                term = compare_term(ta, terms, args_utr.fuzzy)
            else:
                term = None
            attributes = []
            if term_file is not None:
                if term is not None:
                    attributes.append(
                        "=".join(["associated_term", "Terminator:" +
                                  str(term.start) + "-" +
                                  str(term.end) + "_" + term.strand]))
                else:
                    attributes.append("=".join(["associated_term", "NA"]))
            near_cds = get_near_cds(cdss, genes, ta, attributes)
            if ta.seq_id != pre_seq_id:
                pre_seq_id = ta.seq_id
                utr_strain[ta.seq_id] = []
            num_utr = get_3utr(ta, near_cds, utr_all, utr_strain,
                               attributes, num_utr, out, args_utr)
    else:
        compare_term_3utr(terms, cdss, genes, utr_all,
                          utr_strain, args_utr, out)
    name = (gff_file.split("/"))[-1].replace(".gff", "")
    plot(utr_all, None, None, "_".join([name, "all_3utr_length.png"]),
         None, "3utr", "3utr")
    out.close()
    if len(utr_strain) > 1:
        for strain in utr_strain.keys():
            plot(utr_strain[strain], None, None,
                 "_".join([strain, "3utr_length.png"]), None, "3utr", "3utr")



import os
import sys
import time
import shutil
from glob import glob
from subprocess import call, Popen
from annogesiclib.multiparser import Multiparser
from annogesiclib.helper import Helper
from annogesiclib.converter import Converter
from annogesiclib.circRNA import detect_circrna


class CircRNADetection(object):
    '''Detection of circRNA'''

    def __init__(self, args_circ):
        self.multiparser = Multiparser()
        self.helper = Helper()
        self.converter = Converter()
        self.alignment_path = os.path.join(args_circ.output_folder,
                                           "segemehl_align")
        self.splice_path = os.path.join(args_circ.output_folder,
                                        "segemehl_splice")
        self.candidate_path = os.path.join(args_circ.output_folder,
                                           "circRNA_tables")
        self.gff_folder = os.path.join(args_circ.output_folder, "gffs")
        self.gff_path = os.path.join(args_circ.gffs, "tmp")
        self.splices = {"all_file": "splicesites_all.bed",
                        "file": "splicesites.bed",
                        "all": "splicesites_all", "splice": "splicesites"}
        self.trans = {"all_file": "transrealigned_all.bed",
                      "file": "transrealigned.bed",
                      "all": "transrealigned_all", "trans": "transrealigned"}
        self.bams = {"whole": "whole_reads.bam", "sort": "whole_reads_sort"}
        if args_circ.align:
            if args_circ.fastas is None:
                print("Error: There is no genome fasta file!!!")
                sys.exit()
            else:
                self.fasta_path = os.path.join(args_circ.fastas, "tmp")
        else:
            self.fasta_path = os.path.join(args_circ.fastas, "tmp")

    def _wait_process(self, processes):
        '''wait for the parallels to finish the process'''
        for p in processes:
            p.wait()
            if p.stdout:
                p.stdout.close()
            if p.stdin:
                p.stdin.close()
            if p.stderr:
                p.stderr.close()
            try:
                p.kill()
            except OSError:
                pass
            time.sleep(5)

    def _deal_zip_file(self, read_files):
        tmp_reads = []
        for reads in read_files:
            for read in glob(reads):
                if read.endswith(".bz2"):
                    mod_read = read.replace(".bz2", "")
                    if (".fa" not in mod_read) and (
                            ".fasta" not in mod_read) and (
                            ".fna" not in mod_read):
                        mod_read = mod_read + ".fa"
                    read_out = open(mod_read, "w")
                    tmp_reads.append(mod_read)
                    print(" ".join(["Unzipping", read]))
                    call(["bzcat", read], stdout=read_out)
                    read_out.close()
                elif read.endswith(".gz"):
                    mod_read = read.replace(".gz", "")
                    if (".fa" not in mod_read) and (
                            ".fasta" not in mod_read) and (
                            ".fna" not in mod_read):
                        mod_read = mod_read + ".fa"
                    read_out = open(mod_read, "w")
                    tmp_reads.append(mod_read)
                    print(" ".join(["Unzipping", read]))
                    call(["zcat", read], stdout=read_out)
                    read_out.close()
        return tmp_reads

    def _run_segemehl_fasta_index(self, segemehl_path, fasta_path,
                                  index, fasta):
        call([segemehl_path,
              "-x", os.path.join(fasta_path, index),
              "-d", os.path.join(fasta_path, fasta)])

    def _run_segemehl_align(self, args_circ, index, fasta, read,
                            sam_file, log_file, fasta_prefix):
        out = open(os.path.join(self.alignment_path,
                   fasta_prefix, sam_file), "w")
        log = open(os.path.join(self.alignment_path,
                   fasta_prefix, log_file), "w")
        p = Popen([args_circ.segemehl_path,
                   "-i", os.path.join(self.fasta_path, index),
                   "-d", os.path.join(self.fasta_path, fasta),
                   "-q", read, "-S"],
                  stdout=out, stderr=log)
        return p

    def _align(self, args_circ):
        '''align the read. if the bam files are provided, it can be skipped.'''
        prefixs = []
        align_files = []
        for fasta in os.listdir(self.fasta_path):
            index = fasta.replace(".fa", ".idx")
            self._run_segemehl_fasta_index(args_circ.segemehl_path,
                                           self.fasta_path, index, fasta)
            processes = []
            num_process = 0
            fasta_prefix = fasta.replace(".fa", "")
            prefixs.append(fasta_prefix)
            self.helper.check_make_folder(os.path.join(
                                self.alignment_path, fasta_prefix))
            for reads in args_circ.read_files:
                for read in glob(reads):
                    num_process += 1
                    read_name = read.split("/")[-1]
                    if read_name.endswith(".fa") or \
                       read_name.endswith(".fna") or \
                       read_name.endswith("fasta"):
                        filename = read_name.split(".")
                        read_prefix = ".".join(filename[:-1])
                        sam_file = "_".join([read_prefix, fasta_prefix + ".sam"])
                        log_file = "_".join([read_prefix, fasta_prefix + ".log"])
                        align_files.append("_".join([read_prefix, fasta_prefix]))
                        print("Mapping {0}".format(sam_file))
                        p = self._run_segemehl_align(
                                args_circ, index, fasta, read,
                                sam_file, log_file, fasta_prefix)
                        processes.append(p)
                        if num_process == args_circ.cores:
                            self._wait_process(processes)
                            num_process = 0
            self._wait_process(processes)
        return align_files, prefixs

    def _run_samtools_convert_bam(self, samtools_path, pre_sam, out_bam):
        call([samtools_path, "view", "-bS", pre_sam, "-o", out_bam])

    def _convert_sam2bam(self, sub_alignment_path, samtools_path, align_files):
        bam_files = []
        convert_ones = []
        remove_ones = []
        for sam in os.listdir(sub_alignment_path):
            pre_sam = os.path.join(sub_alignment_path, sam)
            if sam.endswith(".sam"):
                bam_file = sam.replace(".sam", ".bam")
                print("Converting {0} to {1}".format(sam, bam_file))
                out_bam = os.path.join(sub_alignment_path, bam_file)
                self._run_samtools_convert_bam(samtools_path, pre_sam, out_bam)
                bam_files.append(out_bam)
                if align_files:
                    if bam_file.replace(".bam", "") not in align_files:
                        convert_ones.append(out_bam)
                    else:
                        remove_ones.append(pre_sam)
            elif sam.endswith(".bam"):
                if (pre_sam not in convert_ones) and (
                        pre_sam not in remove_ones):
                    bam_files.append(pre_sam)
            elif sam.endswith(".log"):
                os.remove(pre_sam)
        return bam_files, convert_ones, remove_ones

    def _run_samtools_merge_sort(self, samtools_path,
                                 out_folder, bam_files):
        print("Merging all bam files")
        whole_bam = os.path.join(out_folder, self.bams["whole"])
        if len(bam_files) <= 1:
            shutil.copyfile(bam_files[0], whole_bam)
        else:
            file_line = " ".join(bam_files)
            os.system(" ".join([samtools_path, "merge",
                                whole_bam, file_line]))
        print("Sorting bam files")
        call([samtools_path, "sort", "-o", os.path.join(out_folder,
              self.bams["sort"] + ".bam"), whole_bam])
        os.remove(os.path.join(out_folder, self.bams["whole"]))

    def _run_samtools_convert_sam(self, samtools_path, out_folder):
        print("Converting whole reads bam file to sam file")
        call([samtools_path, "view", "-h", "-o",
              os.path.join(out_folder, self.bams["sort"] + ".sam"),
              os.path.join(out_folder, self.bams["sort"] + ".bam")])

    def _merge_sort_aligment_file(self, bam_files, samtools_path,
                                  out_folder, convert_ones,
                                  tmp_reads, remove_ones):
        self._run_samtools_merge_sort(samtools_path,
                                      out_folder, bam_files)
        self._run_samtools_convert_sam(samtools_path, out_folder)
        for bam in convert_ones:
            os.remove(bam)
        for sam in remove_ones:
            os.remove(sam)
        if len(tmp_reads) != 0:
            for read in tmp_reads:
                os.remove(read)

    def _run_testrealign(self, prefix, testrealign_path, out_folder):
        self.helper.check_make_folder(os.path.join(self.splice_path, prefix))
        sub_splice_path = os.path.join(self.splice_path, prefix)
        err_log = os.path.join(sub_splice_path, prefix + ".log")
        print("Running testrealign.x for {0}".format(prefix))
        command = " ".join([
                  testrealign_path,
                  "-d", os.path.join(self.fasta_path, prefix + ".fa"),
                  "-q", os.path.join(out_folder, self.bams["sort"] + ".sam"),
                  "-n"])
        os.system(command + " 2>" + err_log)
        self.helper.move_all_content(os.getcwd(), sub_splice_path, [".bed"])
        self.helper.remove_all_content(out_folder, self.bams["sort"], "file")

    def _merge_bed(self, fastas, splice_path):
        '''Merge the bed files for analysis'''
        tmp_prefixs = []
        for fasta in os.listdir(fastas):
            headers = []
            if (fasta.endswith(".fa") or fasta.endswith(".fna") or
                    fasta.endswith(".fasta")):
                with open(os.path.join(fastas, fasta), "r") as f_h:
                    for line in f_h:
                        line = line.strip()
                        if line.startswith(">"):
                            headers.append(line[1:])
                filename = fasta.split(".")
                fasta_prefix = ".".join(filename[:-1])
                tmp_prefixs.append(fasta_prefix)
                self.helper.check_make_folder(os.path.join(
                                              os.getcwd(), fasta_prefix))
                for header in headers:
                    shutil.copyfile(os.path.join(splice_path, header,
                                    self.splices["file"]),
                                    os.path.join(fasta_prefix,
                                    "_".join([self.splices["splice"],
                                              header + ".bed"])))
                    shutil.copyfile(os.path.join(splice_path, header,
                                    self.trans["file"]),
                                    os.path.join(fasta_prefix,
                                    "_".join([self.trans["trans"],
                                              header + ".bed"])))
                out_splice = os.path.join(fasta_prefix,
                                          self.splices["all_file"])
                out_trans = os.path.join(fasta_prefix,
                                         self.trans["all_file"])
                if len(headers) > 1:
                    for file_ in os.listdir(fasta_prefix):
                        if (self.splices["splice"] in file_) and (
                                self.splices["all"] not in file_):
                            self.helper.merge_file(os.path.join(
                                    fasta_prefix, file_), out_splice)
                        elif (self.trans["trans"] in file_) and (
                                self.trans["all"] not in file_):
                            self.helper.merge_file(os.path.join(
                                    fasta_prefix, file_), out_trans)
                else:
                    shutil.move(os.path.join(
                                fasta_prefix,
                                "_".join([self.splices["splice"],
                                         headers[0] + ".bed"])),
                                out_splice)
                    shutil.move(os.path.join(
                                fasta_prefix,
                                "_".join([self.trans["trans"],
                                          headers[0] + ".bed"])),
                                out_trans)
        self.helper.remove_all_content(splice_path, None, "dir")
        return tmp_prefixs

    def _stat_and_gen_gff(self, tmp_prefixs, args_circ):
        '''do statistics and print the result to gff file'''
        for prefix in tmp_prefixs:
            self.helper.check_make_folder(os.path.join(self.gff_folder,
                                                       prefix))
            shutil.copytree(prefix, os.path.join(self.splice_path, prefix))
            self.helper.check_make_folder(os.path.join(
                                          self.candidate_path, prefix))
            print("Comparing with annotation of {0}".format(prefix))
            if self.splices["all_file"] in os.listdir(os.path.join(
                                           self.splice_path, prefix)):
                detect_circrna(os.path.join(self.splice_path, prefix,
                               self.splices["all_file"]), os.path.join(
                               self.gff_path, prefix + ".gff"),
                               os.path.join(self.candidate_path, prefix,
                               "_".join([prefix, "circRNA_all.csv"])),
                               args_circ, os.path.join(args_circ.stat_folder,
                               "_".join(["stat_circRNA", prefix + ".csv"])))
                self.converter.convert_circ2gff(
                     os.path.join(self.candidate_path, prefix,
                                  "_".join([prefix, "circRNA_all.csv"])),
                     args_circ, os.path.join(
                                self.gff_folder, prefix,
                                "_".join([prefix, "circRNA_all.gff"])),
                     os.path.join(self.gff_folder, prefix,
                                  "_".join([prefix, "circRNA_best.gff"])))

    def _get_bams(self, bam_files):
        filenames = []
        for bams in bam_files:
            for bam in glob(bams):
                filenames.append(bam)
        return filenames

    def run_circrna(self, args_circ):
        '''detection of circRNA'''
        for gff in os.listdir(args_circ.gffs):
            if gff.endswith(".gff"):
                self.helper.check_uni_attributes(os.path.join(
                                                 args_circ.gffs, gff))
        if args_circ.segemehl_path is None:
            print("Error: please assign segemehl folder!!")
            sys.exit()
        self.multiparser.parser_gff(args_circ.gffs, None)
        self.multiparser.combine_gff(args_circ.fastas, self.gff_path,
                                     "fasta", None)
        tmp_reads = []
        if args_circ.align:
            self.multiparser.parser_fasta(args_circ.fastas)
            tmp_reads = self._deal_zip_file(args_circ.read_files)
            align_files, prefixs = self._align(args_circ)
        else:
            self.multiparser.parser_fasta(args_circ.fastas)
            prefixs = []
            for fasta in os.listdir(self.fasta_path):
                fasta_prefix = fasta.replace(".fa", "")
                prefixs.append(fasta_prefix)
            bam_files = self._get_bams(args_circ.bams)
            align_files = None
        for prefix in prefixs:
            if args_circ.align:
                sub_alignment_path = os.path.join(self.alignment_path, prefix)
                bam_files, convert_ones, remove_ones = self._convert_sam2bam(
                    sub_alignment_path, args_circ.samtools_path, align_files)
            else:
                convert_ones = []
                remove_ones = []
            self._merge_sort_aligment_file(
                bam_files, args_circ.samtools_path, args_circ.output_folder,
                convert_ones, tmp_reads, remove_ones)
            self._run_testrealign(prefix, args_circ.testrealign_path,
                                  args_circ.output_folder)
        tmp_prefixs = self._merge_bed(args_circ.fastas, self.splice_path)
        self.multiparser.parser_gff(args_circ.gffs, None)
        self.multiparser.combine_gff(args_circ.fastas, self.gff_path,
                                     "fasta", None)
        self._stat_and_gen_gff(tmp_prefixs, args_circ)
        self.helper.remove_tmp_dir(args_circ.fastas)
        self.helper.remove_tmp_dir(args_circ.gffs)
        for tmp_prefix in tmp_prefixs:
            shutil.rmtree(tmp_prefix)
#        if (not args_circ.align) and (len(remove_frag) != 0):
#            for frag in remove_frag:
#                os.remove(os.path.join(merge_folder, frag))



import os
import csv
from annogesiclib.gff3 import Gff3Parser


def output_coverage(table_file, gff_file, cutoff_cover, stat_file, out_folder):
    out = open(os.path.join(out_folder, "tmp_srna_table"), "w")
    out_g = open(os.path.join(out_folder, "tmp_srna_gff"), "w")
    out.write("\t".join([
        "rank", "strain", "name", "start", "end", "strand",
        "start_with_TSS/Cleavage_site", "end_with_cleavage", "candidates",
        "lib_type", "best_avg_coverage", "best_highest_coverage",
        "best_lower_coverage", "track/coverage",
        "normalized_secondary_energy_change(by_length)",
        "UTR_derived/Intergenic", "confliction_of_sORF", "nr_hit_number",
        "sRNA_hit_number", "nr_hit_top3|ID|e-value", "sRNA_hit|e-value",
        "overlap_CDS", "overlap_percent", "end_with_terminator"]) + "\n")
    out_g.write("##gff-version 3\n")
    stat_out = open(stat_file, "w")
    nums = {5: 0}
    for i in range(10, 100, 10):
        nums[i] = 0
    for i in range(100, 1000, 100):
        nums[i] = 0
    for i in range(1000, 5000, 500):
        nums[i] = 0
    gffs = []
    gh = open(gff_file, "r")
    for entry in Gff3Parser().entries(gh):
        gffs.append(entry)
    fh = open(table_file, "r")
    rank = 1
    new_gffs = []
    for row in csv.reader(fh, delimiter='\t'):
        if row[0] != "rank":
            for cutoff in nums.keys():
                if float(row[10]) >= cutoff:
                    nums[cutoff] += 1
            if float(row[10]) >= cutoff_cover:
                row[0] = str(rank)
                out.write("\t".join(row) + "\n")
                rank += 1
                for gff in gffs:
                    if (row[1] == gff.seq_id) and (
                            row[3] == str(gff.start)) and (
                            row[4] == str(gff.end)) and (
                            row[5] == gff.strand):
                        new_gffs.append(gff)
    sort_gffs = sorted(new_gffs, key=lambda k: (k.seq_id, k.start,
                                                k.end, k.strand))
    for gff in sort_gffs:
        out_g.write(gff.info + "\n")
    coverlist = sorted(nums, key=lambda key: nums[key])
    stat_out.write("coverage\tfrequency\n")
    for cover in coverlist:
        stat_out.write("\t".join([str(cover), str(nums[cover])]) + "\n")



import math
from annogesiclib.gff3 import Gff3Parser
from annogesiclib.helper import Helper


def import_candidate(cands, term_features, strain, start, end, ut, name,
                     total_length, strand, parent_p, parent_m, p_pos, m_pos):
    cands.append({"strain": strain, "start": start, "end": end, "print": False,
                  "ut": ut, "name": name, "miss": term_features["real_miss"],
                  "loop": term_features["loop"], "length": total_length,
                  "r_stem": term_features["r_stem"], "strand": strand,
                  "l_stem": term_features["l_stem"], "parent_p": parent_p,
                  "parent_m": parent_m, "detect_p": False, "detect_m": False,
                  "p_pos": p_pos, "m_pos": m_pos})


def get_feature(gene):
    if "Name" in gene.attributes.keys():
        feature = gene.attributes["Name"]
    elif "locus_tag" in gene.attributes.keys():
        feature = gene.attributes["locus_tag"]
    else:
        strand = Helper().get_strand_name(gene.strand)
        feature = "".join([gene.feature, ":", str(gene.start),
                           "-", str(gene.end), "_", strand])
    return feature


def check_miss(cand1, cand2, cutoff_miss):
    stem_len = (cand2["r_stem"] + cand2["l_stem"] - cand2["miss"])
    if (float(cand2["miss"]) / float(stem_len)) <= cutoff_miss:
        cand1["miss"] = cand2["miss"]
        cand1["r_stem"] = cand2["r_stem"]
        cand1["l_stem"] = cand2["l_stem"]


def filter_term(cands, terms, miss_rate):
    '''remove the low possibilty terminator'''
    cutoff_miss = miss_rate
    for cand1 in cands:
        stem_len = (cand1["r_stem"] + cand1["l_stem"] - cand1["miss"])
        if (not cand1["print"]) and \
           ((float(cand1["miss"]) / float(stem_len)) <= cutoff_miss):
            tmp_term = cand1.copy()
            for cand2 in cands:
                if (tmp_term["strain"] == cand2["strain"]) and (
                        tmp_term["miss"] >= cand2["miss"]):
                    if (tmp_term["start"] >= cand2["start"]) and (
                            tmp_term["start"] < cand2["end"]) and (
                            tmp_term["end"] > cand2["end"]):
                        tmp_term["start"] = cand2["start"]
                        check_miss(tmp_term, cand2, cutoff_miss)
                        cand2["print"] = True
                    elif (cand2["start"] > tmp_term["start"]) and (
                            cand2["start"] < tmp_term["end"]) and (
                            cand2["end"] >= tmp_term["end"]):
                        tmp_term["end"] = cand2["end"]
                        check_miss(tmp_term, cand2, cutoff_miss)
                        cand2["print"] = True
                    elif (tmp_term["start"] >= cand2["start"]) and (
                            tmp_term["end"] <= cand2["end"]):
                        tmp_term["start"] = cand2["start"]
                        tmp_term["end"] = cand2["end"]
                        check_miss(tmp_term, cand2, cutoff_miss)
                        cand2["print"] = True
                    elif (cand2["start"] >= tmp_term["start"]) and (
                            cand2["end"] <= tmp_term["end"]):
                        cand2["print"] = True
                        check_miss(tmp_term, cand2, cutoff_miss)
            terms.append(tmp_term)


def check_sec(sec, nts):
    '''check the criteria of sec str of terminator'''
    term_features = {"st_pos": 0, "rights": 0, "lefts": 0,
                     "tmp_miss": 0, "real_miss": 0, "loop": 0,
                     "r_stem": 0, "l_stem": 0}
    detects = {"detect_r": False, "detect_l": False,
               "conflict": False}
    for s_t in reversed(sec[0:nts]):
        term_features["st_pos"] += 1
        if s_t == ")":
            if not detects["detect_l"]:
                term_features["rights"] += 1
                term_features["real_miss"] = term_features["tmp_miss"]
                detects["detect_r"] = True
            else:
                detects["conflict"] = True
                break
        elif s_t == ".":
            if detects["detect_r"]:
                term_features["tmp_miss"] += 1
        elif s_t == "(":
            term_features["lefts"] += 1
            if not detects["detect_l"]:
                term_features["loop"] = (
                    term_features["tmp_miss"] - term_features["real_miss"])
                term_features["tmp_miss"] = term_features["real_miss"]
                term_features["r_stem"] = (
                    term_features["rights"] + term_features["real_miss"])
            else:
                term_features["real_miss"] = term_features["tmp_miss"]
            detects["detect_l"] = True
            if term_features["lefts"] == term_features["rights"]:
                break
    return term_features, detects


def detect_candidates(seq, sec, name, strain, start, end, parent_p, parent_m,
                      strand, args_term, p_pos, m_pos):
    '''check the criteria of sec str of terminator'''
    term_len = 2 * args_term.max_stem + 2 * (
               args_term.max_stem * args_term.miss_rate) + args_term.max_loop
    cands = []
    for nts in range(0, len(seq) - 6):
        ut = 0
        for nt in seq[nts:nts + args_term.range_u]:
            if (nt == "U") or (nt == "T"):
                ut += 1
        if (ut >= args_term.at_tail) and (nts > 10):
            if sec[nts - 1] == ")":
                term_features = {"st_pos": 0, "rights": 0, "lefts": 0,
                                 "tmp_miss": 0, "real_miss": 0, "loop": 0,
                                 "r_stem": 0, "l_stem": 0}
                detects = {"detect_r": False, "detect_l": False,
                           "conflict": False}
                term_features, detects = check_sec(sec, nts)
                if detects["conflict"] is False:
                    total_length = (
                        (nts) - (nts - term_features["st_pos"] + 1) + 1)
                    term_features["l_stem"] = (
                        total_length - term_features["r_stem"] -
                        term_features["loop"])
                    if (total_length <= term_len) and (
                            term_features["loop"] <= args_term.max_loop) and (
                            term_features["loop"] >= args_term.min_loop) and (
                            ((term_features["r_stem"] +
                              term_features["l_stem"] -
                              term_features["real_miss"]) / 2) >=
                            args_term.min_stem) and (
                            ((term_features["r_stem"] +
                              term_features["l_stem"] -
                              term_features["real_miss"]) / 2) <=
                            args_term.max_stem):
                        if strand == "+":
                            import_candidate(
                                cands, term_features, strain,
                                start + (nts - term_features["st_pos"]) - 10,
                                start + nts - 1 + 10, ut, name, total_length,
                                strand, parent_p, parent_m, p_pos, m_pos)
                        else:
                            import_candidate(
                                cands, term_features, strain,
                                end - (nts - 1) - 10,
                                end - (nts - term_features["st_pos"]) + 10,
                                ut, name, total_length, strand,
                                parent_p, parent_m, p_pos, m_pos)
    return cands


def check_parent(genes, term, detects, strand, fuzzy_up, fuzzy_down, type_):
    tmp = None
    for gene in genes:
        if (term["strain"] == gene.seq_id) and (
                gene.strand == strand):
            if type_ == "parent_p":
                if ((term["start"] - fuzzy_down) <= gene.end) and (
                        term["start"] >= gene.end):
                    detects[type_] = True
                    tmp = get_feature(gene)
                elif ((gene.end - term["end"]) <= fuzzy_up) and (
                        (gene.end - term["end"]) >= 0):
                    detects[type_] = True
                    tmp = get_feature(gene)
                elif ((gene.end - term["start"]) > fuzzy_up) and (
                        (gene.end - term["start"]) >= 0):
                    break
            elif type_ == "parent_m":
                if ((term["end"] + fuzzy_down) >= gene.start) and (
                        term["end"] <= gene.start):
                    detects[type_] = True
                    tmp = get_feature(gene)
                elif ((term["start"] - gene.start) <= fuzzy_up) and (
                        (term["start"] - gene.start) >= 0):
                    detects[type_] = True
                    tmp = get_feature(gene)
                elif (gene.start - term["end"] > fuzzy_down):
                    break
    return tmp


def parents(terms, genes, args_term):
    '''assign the associated gene to terminator'''
    for term in terms:
        detects = {"parent_p": False, "parent_m": False}
        if "tran" in term["parent_p"]:
            tmp_p = check_parent(genes, term, detects, "+",
                                 args_term.fuzzy_up_gene,
                                 args_term.fuzzy_down_gene, "parent_p")
#            pos = term["parent_p"].split(":")[-1].split("_")[0].split("-")[-1]
            pos = term["p_pos"].split("-")[-1]
            if ((term["start"] - int(pos) <= args_term.fuzzy_down_ta) and (
                 term["start"] - int(pos) >= 0)) or (
                (int(pos) - term["end"] <= args_term.fuzzy_up_ta) and (
                 int(pos) - term["end"] >= 0)):
                pass
            else:
                term["parent_p"] = ""
        if "tran" in term["parent_m"]:
            tmp_m = check_parent(genes, term, detects, "-",
                                 args_term.fuzzy_up_gene,
                                 args_term.fuzzy_down_gene, "parent_m")
            pos = term["m_pos"].split("-")[0]
#            pos = term["parent_m"].split(":")[-1].split("_")[0].split("-")[0]
            if ((int(pos) - term["end"] <= args_term.fuzzy_down_ta) and (
                 int(pos) - term["end"] >= 0)) or (
                (term["start"] - int(pos) <= args_term.fuzzy_up_ta) and (
                 term["start"] - int(pos) >= 0)):
                pass
            else:
                term["parent_m"] = ""
        if detects["parent_p"]:
            if len(term["parent_p"]) == 0:
                term["parent_p"] = tmp_p
            else:
                term["parent_p"] = ",".join([term["parent_p"], tmp_p])
        if detects["parent_m"]:
            if len(term["parent_m"]) == 0:
                term["parent_m"] = tmp_m
            else:
                term["parent_m"] = ",".join([term["parent_m"], tmp_m])


def read_gff(seq_file, gff_file, tran_file):
    genome = {}
    genes = []
    trans = []
    for entry in Gff3Parser().entries(open(gff_file)):
        if (entry.feature == "gene"):
            genes.append(entry)
    for entry in Gff3Parser().entries(open(tran_file)):
        trans.append(entry)
    with open(seq_file, "r") as q_h:
        for line in q_h:
            line = line.strip()
            if line.startswith(">"):
                strain = line[1:]
                genome[strain] = ""
            else:
                genome[strain] = genome[strain] + line
    genes = sorted(genes, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    trans = sorted(trans, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    return genes, genome, trans


def compare_anno(gffs, cands, fuzzy_up, fuzzy_down):
    '''compare the terminator with CDS'''
    detect = False
    new_cands = []
    for cand in cands:
        for gff in gffs:
            if (gff.seq_id == cand["strain"]) and (
                    gff.strand == cand["strand"]):
                if cand["strand"] == "+":
                    if (gff.start <= cand["start"]) and (
                            gff.end >= cand["start"]) and (
                            gff.end <= cand["end"]):
                        detect = True
                        break
                    elif (math.fabs(gff.end - cand["end"]) <= fuzzy_up) and (
                            gff.end >= cand["end"]):
                        detect = True
                        break
                    elif (math.fabs(gff.end - cand["start"]) <=
                            fuzzy_down) and (gff.end <= cand["start"]):
                        detect = True
                        break
                else:
                    if (gff.start >= cand["start"]) and (
                            gff.start <= cand["end"]) and (
                            gff.end >= cand["end"]):
                        detect = True
                        break
                    elif (math.fabs(gff.start - cand["end"]) <=
                            fuzzy_down) and (gff.start >= cand["end"]):
                        detect = True
                        break
                    elif (math.fabs(gff.start - cand["start"]) <=
                            fuzzy_up) and (cand["start"] >= gff.start):
                        detect = True
                        break
        if detect:
            detect = False
            new_cands.append(cand)
    return new_cands


def merge_cands(new_cands_gene, new_cands_ta):
    new_cands = []
    for cand_gene in new_cands_gene:
        new_cands.append(cand_gene)
    for cand_ta in new_cands_ta:
        if cand_ta not in new_cands:
            new_cands.append(cand_ta)
    new_cands = sorted(new_cands, key=lambda k: (k["strain"], k["start"],
                                                 k["end"], k["strand"]))
    return new_cands


def get_seq_sec(s_h, sec_seq):
    '''extract the secondary structure information'''
    for line in s_h:
        if ("(" in line) or ("." in line) or (")" in line):
            line = line.split(" ")
            sec_seq["sec"] = line[0]
            break
        else:
            sec_seq["seq"] = line


def print_term(terms, out):
#    First = True
    for term in terms:
#        if First:
#            Fisrt = False
#            pre_term = term
#        else:
#            if (term["strain"] == pre_term["strain"]) and (
#                    term["strand"] == pre_term["strand"]):
#                if ((term["start"] >= pre_term["start"]) and (
#                        term["end"] <= pre_term["end"])) or (
#                        (term["start"] <= pre_term["start"]) and (
#                        term["end"] >= pre_term["end"])) or (
#                        (term["start"] >= pre_term["start"]) and (
#                        term["start"] <= pre_term["end"]) and (
#                        term["end"] >= pre_term["end"])) or (
#                        (term["start"] <= pre_term["start"]) and (
#                        term["end"] >= pre_term["start"]) and (
#                        term["end"] <= pre_term["end"])):
        print_ = False
        if (term["strand"] == "+") and (len(term["parent_p"]) != 0):
            print_ = True
        elif (term["strand"] == "-") and (len(term["parent_m"]) != 0):
            print_ = True
        if print_: 
            out.write("\t".join([term["strain"], str(term["start"]),
                      str(term["end"]), term["name"], str(term["miss"]),
                      str(term["loop"]), str(term["length"]),
                      str(term["r_stem"]), term["strand"], str(term["l_stem"]),
                      term["parent_p"], term["parent_m"],
                      str(term["ut"])]) + "\n")


def poly_t(seq_file, sec_file, gff_file, tran_file, out_file, args_term):
    '''detect the sec str of terminator'''
    terms = []
    genes, genome, trans = read_gff(seq_file, gff_file, tran_file)
    out = open(out_file, "w")
    with open(sec_file, "r") as s_h:
        for line in s_h:
            line = line.strip()
            if line.startswith(">"):
                line = line[1:]
                name = line.split("|")[0]
                start = int(line.split("|")[1])
                end = int(line.split("|")[2])
                strain = line.split("|")[3]
                parent_p = line.split("|")[4]
                parent_m = line.split("|")[5]
                p_pos = line.split("|")[6]
                m_pos = line.split("|")[7]
                strand = line.split("|")[-1]
                sec_seq = {"sec": "", "seq": ""}
                get_seq_sec(s_h, sec_seq)
                if len(sec_seq["seq"]) <= 6:
                    continue
                else:
                    cands = detect_candidates(
                        sec_seq["seq"], sec_seq["sec"], name, strain, start,
                        end, parent_p, parent_m, strand, args_term,
                        p_pos, m_pos)
                cands = sorted(cands, key=lambda x: (x["miss"], x["start"]))
                new_cands_gene = compare_anno(genes, cands,
                                              args_term.fuzzy_up_gene,
                                              args_term.fuzzy_down_gene)
                new_cands_ta = compare_anno(trans, cands,
                                            args_term.fuzzy_up_ta,
                                            args_term.fuzzy_down_ta)
                new_cands = merge_cands(new_cands_gene, new_cands_ta)
                filter_term(new_cands, terms, args_term.miss_rate)
    parents(terms, genes, args_term)
    print_term(terms, out)



import os
import csv
import shutil


def print_rank_all(all_table, best_table):
    out = open("tmp_rank_table", "w")
    fh = open(best_table, "r")
    rank = 0
    bests = []
    for row in csv.reader(fh, delimiter='\t'):
        if row[0] != "rank":
            bests.append(row)
            rank = int(row[0])
        out.write("\t".join(row) + "\n")
    fh.close()
    fh = open(all_table, "r")
    for row in csv.reader(fh, delimiter='\t'):
        detect = False
        if row[0] != "rank":
            for best in bests:
                if (row[1] == best[1]) and (
                        row[3] == best[3]) and (
                        row[4] == best[4]) and (
                        row[5] == best[5]):
                    detect = True
                    break
            if not detect:
                rank += 1
                row[0] = str(rank)
                out.write("\t".join(row) + "\n")
    os.remove(all_table)
    shutil.move("tmp_rank_table", all_table)



import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot
matplotlib.pyplot.style.use('ggplot')


def plot_mountain_plot(input_file, output_name):
    poss = []
    values = []
    check = 0
    pre_check = 0
    f_h = open(input_file, "r")
    while True:
        line = f_h.readline()
        line = line.rstrip()
        if not line:
            matplotlib.pyplot.figure(1)
            matplotlib.pyplot.subplot(212)
            matplotlib.pyplot.xlabel('Nucleotide position')
            matplotlib.pyplot.ylabel('Entropy')
            matplotlib.pyplot.plot(values, color="black")
            matplotlib.pyplot.savefig(output_name, format='pdf')
            break
        elif line == "&":
            line = f_h.readline()
            line = line.rstrip()
            check += 1
        else:
            poss.append(float(line[0:4].replace(" ", "")))
            values.append(float(line[5:].replace(" ", "")))
        if check != pre_check:
            pre_check = check
            if check == 1:
                matplotlib.pyplot.figure(1)
                matplotlib.pyplot.subplot(211)
                ylabel = ("Number of enclosing nucleotides\nor\n"
                          "Min free energy structure")
                matplotlib.pyplot.ylabel(
                    ylabel, fontsize=10, multialignment='left')
                matplotlib.pyplot.plot(values, label='pair probabilities')
                values = []
                poss = []
            elif check == 2:
                matplotlib.pyplot.plot(values, label='mfe structure')
                matplotlib.pyplot.legend(
                    bbox_to_anchor=(0., 1.02, 1., .102),
                    loc=3, ncol=2, mode="expand", borderaxespad=0.)
                values = []
                poss = []
    f_h.close()
    matplotlib.pyplot.cla()
    matplotlib.pyplot.clf()



import os
import sys
from annogesiclib.helper import Helper
from annogesiclib.multiparser import Multiparser
from annogesiclib.optimize_TSSpredator import optimization


def optimize_tss(args_ops):
    if len(os.listdir(args_ops.gffs)) == 0:
        print("Error: There is no gff files!!!")
        sys.exit()
    if len(os.listdir(args_ops.fastas)) == 0:
        print("Error: There is no fasta files!!!")
        sys.exit()
    if len(os.listdir(args_ops.wigs)) == 0:
        print("Error: There is no wiggle files!!!")
        sys.exit()
    Multiparser().parser_wig(args_ops.wigs)
    Multiparser().parser_gff(args_ops.gffs, None)
    Multiparser().parser_fasta(args_ops.fastas)
    gff_path = os.path.join(args_ops.gffs, "tmp")
    wig_path = os.path.join(args_ops.wigs, "tmp")
    fasta_path = os.path.join(args_ops.fastas, "tmp")
    for gff in os.listdir(gff_path):
        if args_ops.project_strain in gff:
            gff_file = os.path.join(gff_path, gff)
            break
    for fa in os.listdir(fasta_path):
        if args_ops.project_strain in fa:
            fasta_file = os.path.join(fasta_path, fa)
            break
    Helper().check_uni_attributes(gff_file)
    optimization(wig_path, fasta_file, gff_file, args_ops)
    Helper().remove_all_content(os.path.join(args_ops.output_folder,
                                "optimized_TSSpredator"), "config", "file")
    Helper().remove_all_content(os.path.join(args_ops.output_folder,
                                "optimized_TSSpredator"), "Master", "dir")
    Helper().remove_tmp_dir(args_ops.wigs)
    Helper().remove_tmp_dir(args_ops.gffs)
    Helper().remove_tmp_dir(args_ops.fastas)



import sys
import os, gc
import numpy as np
from annogesiclib.parser_wig import WigParser


def read_libs(input_libs, wig_folder):
    libs = []
    texs = {}
    for lib in input_libs:
        datas = lib.split(":")
        name = None
        for wig in os.listdir(wig_folder):
            if wig == datas[0]:
                with open(os.path.join(wig_folder, wig), "r") as w_h:
                    for line in w_h:
                        line = line.strip()
                        if line.startswith("track"):
                            name = line.split("=")[-1][1:-1]
                            break
        if name is None:
            print("Error: The {0} can not be found in lib name!!!".format(wig)) 
        if (datas[1] == "tex") or (datas[1] == "notex"):
            cond = "texnotex"
        else:
            cond = datas[1]
        libs.append({"name": name, "type": datas[1],
                     "cond": "_".join([datas[2], cond]),
                     "rep": datas[3], "strand": datas[4]})
    for lib1 in libs:
        if lib1["type"] == "frag":
            pass
        elif (lib1["type"] == "tex") or (lib1["type"] == "notex"):
            prefix1 = lib1["cond"].split("_")[0]
            for lib2 in libs:
                prefix2 = lib2["cond"].split("_")[0]
                if (prefix1 == prefix2) and \
                   (lib1["rep"] == lib2["rep"]) and \
                   (lib1["type"] == "tex") and \
                   (lib2["type"] == "notex") and \
                   (lib1["strand"] == lib2["strand"]):
                    texs[lib1["name"] + "@AND@" + lib2["name"]] = 0
        else:
            print("Error:library type (fragmented or Tex treated??)")
            sys.exit()
    return libs, texs


def read_wig(filename, strand, libs):
    wig_parser = WigParser()
    wigs = {}
    if filename is not False:
        wig_fh = open(filename)
        for entry in wig_parser.parser(wig_fh, strand):
            if entry.strain not in wigs.keys():
                strain = entry.strain
                wigs[strain] = {}
                for lib in libs:
                    if lib["cond"] not in wigs[strain]:
                        wigs[strain][lib["cond"]] = {}
            for lib in libs:
                if (lib["name"] == entry.track) and (
                        lib["strand"] == entry.strand):
                    lib_name = "|".join([
                        entry.track, entry.strand, lib["type"]])
                    if lib_name not in wigs[strain][lib["cond"]].keys():
                        wigs[strain][lib["cond"]][lib_name] = []
                    wigs[strain][lib["cond"]][lib_name].append(entry.coverage)
        wig_fh.close()
        for strain, conds in wigs.items():
            for cond, lib_names in conds.items():
                for lib_name, cover_list in lib_names.items():
                    wigs[strain][cond][lib_name] = np.array(
                            wigs[strain][cond][lib_name])
    return wigs



import shutil
import csv


def import_data(row):
    return{"strain": row[1], "strand": row[2],
           "associate": row[3], "start_seq": int(row[4]),
           "end_seq": int(row[5]), "rfam": row[6], "e": row[7],
           "start_align": int(row[8]), "end_align": int(row[9]),
           "info": row[0:6], "ID": row[0]}


def modify_table(table, output_all):
    first = True
    rbss = []
    out = open("tmp.csv", "w")
    out.write("#ID\tstrain\tstrand\tassociated_CDS\tstart_genome\t"
              "end_genome\tRfam\te_value\tstart_align\tend_align\n")
    if output_all:
        with open(table) as fh:
            for line in fh:
                line = line.strip()
                if first:
                    first = False
                    rbss.append(line)
                    out.write(line + "\n")
                else:
                    if line not in rbss:
                        rbss.append(line)
                        out.write(line + "\n")
    else:
        fh = open(table, "r")
        for row in csv.reader(fh, delimiter='\t'):
            rbss.append(import_data(row))
        for rbs1 in rbss:
            repeat = False
            if "print" not in rbs1.keys():
                rbs1["print"] = True
                for rbs2 in rbss:
                    if (rbs1["strain"] == rbs2["strain"]) and \
                       (rbs1["strand"] == rbs2["strand"]) and \
                       (rbs1["ID"] == rbs2["ID"]):
                        if "print" not in rbs2.keys():
                            rbs2["print"] = True
                            repeat = True
                if not repeat:
                    out.write("\t".join(rbs1["info"] + [rbs1["rfam"],
                                        rbs1["e"], str(rbs1["start_align"]),
                                        str(rbs1["end_align"])]) + "\n")
        fh.close()
    out.close()
    shutil.move("tmp.csv", table)



import os
import shutil
from annogesiclib.gff3 import Gff3Parser
from annogesiclib.helper import Helper


def assign_tss(tss, tran):
    if "ID" in tran.attributes.keys():
        tran_id = tran.attributes["ID"]
    else:
        strand = Helper().get_strand_name(tran.strand)
        tran_id = "".join([tran.feature, ":", str(tran.start), "-",
                           str(tran.end), "_", strand])
    if "Parent" not in tss.attributes.keys():
        tss.attributes["Parent"] = tran_id
    else:
        tss.attributes["Parent"] = \
            ",".join([tss.attributes["Parent"], tran_id])
    if "Name" in tss.attributes.keys():
        tss_name = tss.attributes["Name"]
    else:
        strand = Helper().get_strand_name(tss.strand)
        tss_name = "".join(["TSS:", str(tss.start), "_", strand])
    if "associated_tss" not in tran.attributes.keys():
        tran.attributes["associated_tss"] = tss_name
    else:
        tran.attributes["associated_tss"] = \
            ",".join([tran.attributes["associated_tss"], tss_name])


def del_attributes(entry, features):
    attributes = {}
    for key, value in entry.attributes.items():
        if (key not in features):
            attributes[key] = value
    return attributes


def compare_tran_tss(trans, tsss, fuzzy, stat, out):
    num_tran = 0
    for tran in trans:
        tran.attributes["ID"] = tran.seq_id + "_transcript" + str(num_tran)
        detect = False
        check = [0, 0, 0]
        for tss in tsss:
            if (tss.strand == tran.strand) and (
                    tss.seq_id == tran.seq_id):
                if tss.strand == "+":
                    if (tss.start + int(fuzzy) >= tran.start) and (
                            tss.start <= tran.end):
                        if check[0] != 1:
                            stat["with_TSS"] += 1
                        assign_tss(tss, tran)
                        check[0] = 1
                        detect = True
                        tss.attributes["detect"] = True
                else:
                    if (tss.end - int(fuzzy) <= tran.end) and (
                            tss.end >= tran.start):
                        if check[0] != 1:
                            stat["with_TSS"] += 1
                        assign_tss(tss, tran)
                        check[0] = 1
                        detect = True
                        tss.attributes["detect"] = True
        if not detect:
            stat["no_TSS"] += 1
            tran.attributes["associated_tss"] = "NA"
            check[1] = 1
        else:
            detect = False
        tran.attributes = del_attributes(tran, ["TSS_note", "detect"])
        tran.attribute_string = ";".join(
                ["=".join(items) for items in tran.attributes.items()])
        if out is not None:
            out.write("\t".join([str(field) for field in [
                       tran.seq_id, tran.source, tran.feature, tran.start,
                       tran.end, tran.score, tran.strand, tran.phase,
                       tran.attribute_string + "\n"]]))
        num_tran += 1


def detect_tas_region(tsss, trans, out, out_tss, fuzzy):
    stat = {"with_TSS": 0, "no_TSS": 0, "TSS_no_tran": 0, "TSS_with_tran": 0}
    compare_tran_tss(trans, tsss, fuzzy, stat, out)
    for tss in tsss:
        if "Parent" not in tss.attributes.keys():
            tss.attributes["Parent"] = "NA"
        if ("detect" in tss.attributes.keys()):
            tss.attributes = del_attributes(tss, ["detect"])
            tss.attribute_string = ";".join(
                ["=".join(items) for items in tss.attributes.items()])
            stat["TSS_with_tran"] += 1
            if out_tss is not None:
                out_tss.write("\t".join([str(field) for field in [
                          tss.seq_id, tss.source, tss.feature, tss.start,
                          tss.end, tss.score, tss.strand, tss.phase,
                          tss.attribute_string]]) + "\n")
        else:
            stat["TSS_no_tran"] += 1
            if out_tss is not None:
                out_tss.write("\t".join([str(field) for field in [
                          tss.seq_id, tss.source, tss.feature, tss.start,
                          tss.end, tss.score, tss.strand, tss.phase,
                          tss.attribute_string]]) + "\n")
    return stat


def print_tas_stat(stat, out):
    total_tran = stat["with_TSS"] + stat["no_TSS"]
    total_TSS = stat["TSS_no_tran"] + stat["TSS_with_tran"]
    out.write("\tTranscript starts or overlap with TSS:{0} ({1})\n".format(
              stat["with_TSS"], float(stat["with_TSS"]) / float(total_tran)))
    out.write("\tTranscript has no relationship with TSS:{0} ({1})\n".format(
              stat["no_TSS"], float(stat["no_TSS"]) / float(total_tran)))
    out.write("\tTSS starts or overlap with transcript:{0} ({1})\n".format(
              stat["TSS_with_tran"],
              float(stat["TSS_with_tran"]) / float(total_TSS)))
    out.write("\tTSS has no relationship with transcript:{0} ({1})\n".format(
              stat["TSS_no_tran"],
              float(stat["TSS_no_tran"]) / float(total_TSS)))


def read_tas_file(tss_file, ta_file):
    tsss_uni = {}
    tas_uni = {}
    tsss = []
    tas = []
    tss_f = open(tss_file, "r")
    ta_f = open(ta_file, "r")
    pre_seq_id = ""
    for entry in Gff3Parser().entries(tss_f):
        entry.attributes = del_attributes(entry, ["Parent", "tran_note"])
        if pre_seq_id != entry.seq_id:
            pre_seq_id = entry.seq_id
            tsss_uni[entry.seq_id] = []
        tsss_uni[entry.seq_id].append(entry)
        tsss.append(entry)
    tss_f.close()
    pre_seq_id = ""
    for entry in Gff3Parser().entries(ta_f):
        entry.attributes = del_attributes(entry, ["associated_tss"])
        if pre_seq_id != entry.seq_id:
            pre_seq_id = entry.seq_id
            tas_uni[entry.seq_id] = []
        tas_uni[entry.seq_id].append(entry)
        tas.append(entry)
    ta_f.close()
    tas = sorted(tas, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    return tsss_uni, tsss, tas_uni, tas


def stat_ta_tss(ta_file, tss_file, stat_file, out_ta_file,
                out_tss_file, fuzzy):
    '''statistics for comparison of transcript and TSS'''
    tsss_uni, tsss, tas_uni, tas = read_tas_file(tss_file, ta_file)
    out_stat = open(stat_file, "w")
    out_stat.write("All strains:\n")
    out_ta = open(out_ta_file, "w")
    out_tss = open(out_tss_file, "w")
    out_ta.write("##gff-version 3\n")
    out_tss.write("##gff-version 3\n")
    stats = detect_tas_region(tsss, tas, out_ta, out_tss, fuzzy)
    print_tas_stat(stats, out_stat)
    if (len(tsss_uni) > 1) and (len(tas_uni) > 1):
        for strain_tss in tsss_uni.keys():
            for strain_ta in tas_uni.keys():
                if strain_tss == strain_ta:
                    out_stat.write(strain_tss + ":\n")
                    sort_tas = sorted(tas_uni[strain_ta], key=lambda k: (
                        k.seq_id, k.start, k.end, k.strand))
                    stats = detect_tas_region(tsss_uni[strain_tss],
                                              sort_tas, None, None, fuzzy)
                    print_tas_stat(stats, out_stat)
    out_stat.close()
    out_ta.close()
    out_tss.close()


def assign_parent(gff, tran, feature):
    if "Parent" not in gff.attributes.keys():
        gff.attributes["Parent"] = tran.attributes["ID"]
    else:
        gff.attributes["Parent"] = (
            ",".join([gff.attributes["Parent"], tran.attributes["ID"]]))
    if "_".join(["associated", feature]) not in tran.attributes.keys():
        if "locus_tag" in gff.attributes.keys():
            tran.attributes["_".join(["associated", feature])] = (
                gff.attributes["locus_tag"])
        elif "protein_id" in gff.attributes.keys():
            tran.attributes["_".join(["associated", feature])] = (
                gff.attributes["protein_id"])
        elif "Name" in gff.attributes.keys():
            tran.attributes["_".join(["associated", feature])] = (
                gff.attributes["Name"])
        else:
            strand = Helper().get_strand_name(gff.strand)
            tran.attributes["_".join(["associated", feature])] = (
                "".join([gff.feature, ":", str(gff.start),
                         "-", str(gff.end), "_", strand]))
    else:
        if "locus_tag" in gff.attributes.keys():
            tran.attributes["_".join(["associated", feature])] = (
                ",".join([tran.attributes["_".join(["associated", feature])],
                          gff.attributes["locus_tag"]]))
        elif "protein_id" in gff.attributes.keys():
            tran.attributes["_".join(["associated", feature])] = (
                ",".join([tran.attributes["_".join(["associated", feature])],
                          gff.attributes["protein_id"]]))
        elif "Name" in gff.attributes.keys():
            tran.attributes["_".join(["associated", feature])] = (
                ",".join([tran.attributes["_".join(["associated", feature])],
                          gff.attributes["Name"]]))
        else:
            strand = Helper().get_strand_name(gff.strand)
            tran.attributes["_".join(["associated", feature])] = (
                ",".join([tran.attributes["_".join(
                    ["associated", feature])], "".join(
                        [gff.feature, ":", str(gff.start),
                         "-", str(gff.end), "_", strand])]))


def compare_ta_gff(gffs, tran, check, tran_type, detect, stats, c_feature):
    for gff in gffs:
        if (gff.feature == c_feature):
            if (gff.strand == tran.strand) and (
                    gff.seq_id == tran.seq_id):
                if (gff.start < tran.start) and (
                        gff.end > tran.end):
                    if check[0] != 1:
                        stats[tran.seq_id]["bsae"] += 1
                        stats["All"]["bsae"] += 1
                    tran_type.append("within")
                    assign_parent(gff, tran, c_feature)
                    detect = True
                    check[0] = 1
                elif (gff.start >= tran.start) and (
                        gff.end <= tran.end):
                    if check[3] != 1:
                        stats[tran.seq_id]["asbe"] += 1
                        stats["All"]["asbe"] += 1
                    tran_type.append("cover")
                    assign_parent(gff, tran, c_feature)
                    check[3] = 1
                    detect = True
                elif (gff.start >= tran.start) and (
                        gff.end > tran.end) and (
                        gff.start < tran.end):
                    if check[1] != 1:
                        stats[tran.seq_id]["asae"] += 1
                        stats["All"]["asae"] += 1
                    tran_type.append("left_shift")
                    assign_parent(gff, tran, c_feature)
                    check[1] = 1
                    detect = True
                elif (gff.start < tran.start) and (
                        gff.end <= tran.end) and (
                        gff.end > tran.start):
                    if check[2] != 1:
                        stats[tran.seq_id]["bsbe"] += 1
                        stats["All"]["bsbe"] += 1
                    tran_type.append("right_shift")
                    assign_parent(gff, tran, c_feature)
                    check[2] = 1
                    detect = True
    return detect


def detect_tag_region(gffs, trans, stats, out_t, out_g, c_feature, region):
    detect = False
    for tran in trans:
        check = [0, 0, 0, 0, 0]
        tran_type = []
        tran_type_string = ""
        detect = compare_ta_gff(gffs, tran, check, tran_type,
                                detect, stats, c_feature)
        if not detect:
            stats[tran.seq_id]["other"] += 1
            stats["All"]["other"] += 1
            check[4] = 1
            tran_type.append("not_related")
        else:
            detect = False
        tran_type_string = ",".join(tran_type)
        attribute_string = ";".join(
            ["=".join(items) for items in tran.attributes.items()])
        out_t.write("\t".join(
                    [tran.info_without_attributes, attribute_string]) +
                    ";compare_" + c_feature + "=" + tran_type_string + "\n")
    if region is not None:
        out_g.write(region.info + "\n")
    for gff in gffs:
        attribute_string = ";".join(
            ["=".join(items) for items in gff.attributes.items()])
        out_g.write(gff.info_without_attributes + "\t" +
                    attribute_string + "\n")


def detect_express_gene(gffs, c_feature, strain):
    express_gene = 0
    for gff in gffs:
        if (gff.feature == c_feature) and (
                (strain == "all") or (
                 gff.seq_id == strain)) and (
                "Parent" in gff.attributes.keys()):
            if "tran" in gff.attributes["Parent"].lower():
                express_gene += 1
    return express_gene


def print_tag_stat(stats, out, express_gene, c_feature):
    total = (stats["bsae"] + stats["bsbe"] + stats["asae"] +
             stats["asbe"] + stats["other"])
    out.write("\t\tTranscript starts before and "
              "ends after {0}:{1} ({2})\n".format(
                  c_feature, str(stats["asbe"]),
                  str(float(stats["asbe"]) / float(total))))
    out.write("\t\tTranscript starts after and "
              "ends before {0}:{1} ({2})\n".format(
                  c_feature, str(stats["bsae"]),
                  str(float(stats["bsae"]) / float(total))))
    out.write("\t\tTranscript starts before and "
              "ends within {0}:{1} ({2})\n".format(
                  c_feature, str(stats["asae"]),
                  str(float(stats["asae"]) / float(total))))
    out.write("\t\tTranscript starts within and "
              "ends after {0}:{1} ({2})\n".format(
                  c_feature, str(stats["bsbe"]),
                  str(float(stats["bsbe"]) / float(total))))
    out.write("\t\tTranscript has no overlap of {0}:{1} ({2})\n".format(
                  c_feature, str(stats["other"]),
                  str(float(stats["other"]) / float(total))))
    out.write("\t\tTotal {0}s which have expression:{1} ({2})\n".format(
                  c_feature, str(express_gene),
                  str(float(express_gene) / float(stats["gene"]))))


def read_tag_file(gff_file, ta_file, c_feature):
    region = None
    gffs = []
    tas = []
    stats = {}
    stats["All"] = {"bsae": 0, "bsbe": 0, "asae": 0,
                    "asbe": 0, "other": 0, "gene": 0}
    pre_seq_id = ""
    ta_f = open(ta_file, "r")
    for entry in Gff3Parser().entries(ta_f):
        if entry.seq_id != pre_seq_id:
            pre_seq_id = entry.seq_id
            stats[entry.seq_id] = {"bsae": 0, "bsbe": 0, "asae": 0,
                                   "asbe": 0, "other": 0, "gene": 0}
        entry.attributes = del_attributes(entry, [
                           "_".join(["associated", c_feature]),
                           "_".join(["compare", c_feature])])
        tas.append(entry)
    ta_f.close()
    g_f = open(gff_file, "r")
    for entry in Gff3Parser().entries(g_f):
        if (entry.feature == c_feature):
            ori_parents = []
            if "Parent" in entry.attributes.keys():
                parents = entry.attributes["Parent"].split(",")
                for parent in parents:
                    if "gene" in parent:
                        ori_parents.append(parent)
                if len(ori_parents) == 0:
                    entry.attributes = del_attributes(entry, ["Parent"])
                else:
                    entry.attributes["Parent"] = ",".join(ori_parents)
            if entry.seq_id in stats.keys():
                stats[entry.seq_id]["gene"] += 1
                stats["All"]["gene"] += 1
        if (entry.feature.lower() != "region") and (
                entry.feature.lower() != "source"):
            gffs.append(entry)
        else:
            region = entry
    g_f.close()
    tas = sorted(tas, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    return gffs, tas, stats, region


def stat_ta_gff(ta_file, gff_file, stat_file, out_ta_file, out_gff_file,
                c_feature):
    '''statistics for comparison of transcript and genome annotation'''
    tmp_gff_file = gff_file + "tmp"
    tmp_ta_file = ta_file + "tmp"
    shutil.copy(gff_file, tmp_gff_file)
    shutil.copy(ta_file, tmp_ta_file)
    out_stat = open(stat_file, "w")
    for feature in c_feature:
        og_f = open(out_gff_file, "w")
        o_f = open(out_ta_file, "w")
        o_f.write("##gff-version 3\n")
        og_f.write("##gff-version 3\n")
        gffs, tas, stats, region = read_tag_file(
                tmp_gff_file, tmp_ta_file, feature)
        detect_tag_region(gffs, tas, stats, o_f, og_f, feature, region)
        express_gene = detect_express_gene(gffs, feature, "all")
        out_stat.write("For {0}:\n".format(feature))
        out_stat.write("\tAll strains:\n")
        out_stat.write("\tThe transcriptome assembly information "
                       "compares with {0}:\n".format(feature))
        print_tag_stat(stats["All"], out_stat, express_gene, feature)
        if len(stats) > 2:
            for strain in stats.keys():
                if strain != "All":
                    express_gene = detect_express_gene(gffs, feature, strain)
                    out_stat.write("\t" + strain + ":\n")
                    out_stat.write("\tThe transcriptome assembly information "
                                   "compares with {0}:\n".format(feature))
                    print_tag_stat(stats[strain], out_stat,
                                   express_gene, feature)
        og_f.close()
        o_f.close()
        shutil.copy(out_gff_file, tmp_gff_file)
        shutil.copy(out_ta_file, tmp_ta_file)
    out_stat.close()
    os.remove(tmp_gff_file)
    os.remove(tmp_ta_file)



class Paths(object):
    '''Setup the folders of ANNOgesic'''

    def __init__(self, base_path="."):
        self.base_path = base_path
        self._set_folder_names()

    def _set_folder_names(self):
        """Set the name of folders used in a project."""
        self.input_folder = "%s/input" % (self.base_path)
        self.output_folder = "%s/output" % (self.base_path)
        self._set_input_folder_names()
        self._set_reference_input_folder_names()
        self._set_wig_folder_names()
        self._set_bam_folder_names()
        self._set_output_folder_names()
        self._set_target_folder_names()
        self._set_tsspredator_folder_names()
        self._set_transterm_folder_names()
        self._set_processing_folder_names()
        self._set_transcript_folder_names()
        self._set_utr_folder_names()
        self._set_srna_folder_names()
        self._set_sorf_folder_names()
        self._set_operon_folder_names()
        self._set_circrna_folder_names()
        self._set_goterm_folder_names()
        self._set_starget_folder_names()
        self._set_snp_folder_names()
        self._set_ppi_folder_names()
        self._set_sublocal_folder_names()
        self._set_ribos_folder_names()
        self._set_thermo_folder_names()
        self._set_crispr_folder_names()
        self.version_path = "%s/used_annogesic_version.txt" % (self.base_path)

    def _set_input_folder_names(self):
        self.reference_input_folder = "%s/reference" % self.input_folder
        self.wig_folder = "%s/wigs" % self.input_folder
        self.mutation_table_folder = "%s/mutation_table" % self.input_folder
        self.database_folder = "%s/database" % self.input_folder
        self.manual_TSS_folder = "%s/manual_TSS" % self.input_folder
        self.manual_pro_folder = "%s/manual_processing_site" % (
             self.input_folder)
        self.read_folder = "%s/reads" % self.input_folder
        self.bam_folder = "%s/BAMs" % self.input_folder
        self.riborfam_folder = "%s/riboswitch_ID" % self.input_folder
        self.thermorfam_folder = "%s/RNA_thermometer_ID" % self.input_folder

    def _set_output_folder_names(self):
        self.target_folder = "%s/target" % self.output_folder
        self.ratt_folder = "%s/annotation_transfer" % self.output_folder
        self.tsspredator_folder = "%s/TSS" % self.output_folder
        self.utr_folder = "%s/UTR" % self.output_folder
        self.transterm_folder = "%s/terminator" % self.output_folder
        self.transcript_output_folder = (
                "%s/transcript" % self.output_folder)
        self.processing_site_folder = "%s/processing_site" % self.output_folder
        self.srna_folder = "%s/sRNA" % self.output_folder
        self.sorf_folder = "%s/sORF" % self.output_folder
        self.promoter_output_folder = "%s/promoter_analysis" % (
             self.output_folder)
        self.operon_output_folder = "%s/operons" % self.output_folder
        self.circrna_output_folder = "%s/circRNA" % self.output_folder
        self.goterm_output_folder = "%s/GO_term" % self.output_folder
        self.starget_output_folder = "%s/sRNA_targets" % self.output_folder
        self.snp_output_folder = "%s/SNP_calling" % self.output_folder
        self.ppi_output_folder = "%s/PPI" % self.output_folder
        self.sublocal_output_folder = "%s/subcellular_localization" % (
             self.output_folder)
        self.ribos_output_folder = "%s/riboswitch" % self.output_folder
        self.thermo_output_folder = "%s/RNA_thermometer" % self.output_folder
        self.crispr_output_folder = "%s/crispr" % self.output_folder

    def _set_transcript_folder_names(self):
        self.reference_base_folder = "%s/transcript" % (
             self.output_folder)
        self.transcript_gff_folder = "%s/gffs" % self.reference_base_folder
        self.transcript_stat_folder = "%s/statistics" % (
             self.reference_base_folder)
        self.transcript_table_folder = "%s/tables" % (
             self.reference_base_folder)

    def _set_reference_input_folder_names(self):
        self.reference_base_folder = "%s/reference" % self.input_folder
        self.ref_annotation_folder = "%s/annotation" % (
            self.reference_base_folder)
        self.ref_fasta_folder = "%s/fasta" % (
            self.reference_base_folder)

    def _set_wig_folder_names(self):
        self.wig_base_folder = "%s/wigs" % self.input_folder
        self.frag_folder = "%s/fragment" % (
            self.wig_base_folder)
        self.tex_folder = "%s/tex_notex" % (
            self.wig_base_folder)

    def _set_bam_folder_names(self):
        self.bam_base_folder = "%s/BAMs" % self.input_folder
        self.bam_ref_folder = "%s/BAMs_map_reference" % self.bam_base_folder
        self.bam_tar_folder = "%s/BAMs_map_target" % self.bam_base_folder
        self.bam_ref_frag_folder = "%s/fragment" % (
            self.bam_ref_folder)
        self.bam_tar_frag_folder = "%s/fragment" % (
            self.bam_tar_folder)
        self.bam_ref_tex_folder = "%s/tex_notex" % (
            self.bam_ref_folder)
        self.bam_tar_tex_folder = "%s/tex_notex" % (
            self.bam_tar_folder)

    def _set_target_folder_names(self):
        self.target_base_folder = "%s/target" % self.output_folder
        self.tar_fasta_folder = "%s/fasta" % (
            self.target_base_folder)
        self.tar_annotation_folder = "%s/annotation" % (
            self.target_base_folder)

    def _set_tsspredator_folder_names(self):
        self.tsspredator_base_folder = "%s/TSS" % self.output_folder
        self.tss_to_gff_folder = "%s/gffs" % (
            self.tsspredator_base_folder)
        self.tss_statistics_folder = "%s/statistics" % (
            self.tsspredator_base_folder)
        self.tss_Master_folder = "%s/MasterTables" % (
            self.tsspredator_base_folder)
        self.tss_config_folder = "%s/configs" % (
            self.tsspredator_base_folder)

    def _set_processing_folder_names(self):
        self.processing_base_folder = "%s/processing_site" % self.output_folder
        self.processing_to_gff_folder = "%s/gffs" % (
            self.processing_base_folder)
        self.processing_statistics_folder = "%s/statistics" % (
            self.processing_base_folder)
        self.processing_screenshot_folder = "%s/screenshots" % (
            self.processing_base_folder)
        self.processing_Master_folder = "%s/MasterTables" % (
            self.processing_base_folder)
        self.processing_config_folder = "%s/configs" % (
            self.processing_base_folder)

    def _set_transterm_folder_names(self):
        self.transterm_base_folder = "%s/terminator" % self.output_folder
        self.term_to_gff_folder = "%s/gffs" % (
            self.transterm_base_folder)
        self.term_to_table_folder = "%s/tables" % (
            self.transterm_base_folder)
        self.transtermhp_folder = "%s/transtermhp" % (
            self.transterm_base_folder)
        self.term_statistics_folder = "%s/statistics" % (
            self.transterm_base_folder)

    def _set_utr_folder_names(self):
        self.utr_base_folder = "%s/UTR" % self.output_folder
        self.utr5_folder = "%s/5UTR" % (
            self.utr_base_folder)
        self.utr3_folder = "%s/3UTR" % (
            self.utr_base_folder)
        self.utr3_stat_folder = "%s/statistics" % (
             self.utr3_folder)
        self.utr3_gff_folder = "%s/gffs" % (
             self.utr3_folder)
        self.utr5_stat_folder = "%s/statistics" % (
             self.utr5_folder)
        self.utr5_gff_folder = "%s/gffs" % (
             self.utr5_folder)

    def _set_srna_folder_names(self):
        self.srna_base_folder = "%s/sRNA" % self.output_folder
        self.srna_gff_folder = "%s/gffs" % (
            self.srna_base_folder)
        self.srna_table_folder = "%s/tables" % (
            self.srna_base_folder)
        self.srna_sec_folder = "%s/sec_structure" % (
            self.srna_base_folder)
        self.srna_sec_plot_folder = "%s/sec_structure/sec_plot" % (
            self.srna_base_folder)
        self.srna_dot_plot_folder = "%s/sec_structure/dot_plot" % (
            self.srna_base_folder)
        self.srna_mountain_folder = "%s/mountain_plot" % (
            self.srna_base_folder)
        self.srna_blast_folder = "%s/blast_result_and_misc" % (
            self.srna_base_folder)
        self.srna_stat_folder = "%s/statistics" % (
            self.srna_base_folder)
        self.srna_gff_class_folder = "%s/for_class" % (
            self.srna_gff_folder)
        self.srna_gff_best_folder = "%s/best" % (
            self.srna_gff_folder)
        self.srna_gff_all_folder = "%s/all_candidates" % (
            self.srna_gff_folder)
        self.srna_table_class_folder = "%s/for_class" % (
            self.srna_table_folder)
        self.srna_table_best_folder = "%s/best" % (
            self.srna_table_folder)
        self.srna_table_all_folder = "%s/all_candidates" % (
            self.srna_table_folder)

    def _set_sorf_folder_names(self):
        self.sorf_base_folder = "%s/sORF" % self.output_folder
        self.sorf_gff_folder = "%s/gffs" % (
            self.sorf_base_folder)
        self.sorf_table_folder = "%s/tables" % (
            self.sorf_base_folder)
        self.sorf_stat_folder = "%s/statistics" % (
            self.sorf_base_folder)
        self.sorf_gff_best_folder = "%s/best" % (
            self.sorf_gff_folder)
        self.sorf_gff_all_folder = "%s/all_candidates" % (
            self.sorf_gff_folder)
        self.sorf_table_best_folder = "%s/best" % (
            self.sorf_table_folder)
        self.sorf_table_all_folder = "%s/all_candidates" % (
            self.sorf_table_folder)

    def _set_operon_folder_names(self):
        self.operon_base_folder = "%s/operons" % self.output_folder
        self.operon_gff_folder = "%s/gffs" % (
             self.operon_base_folder)
        self.operon_table_folder = "%s/tables" % (
             self.operon_base_folder)
        self.operon_statistics_folder = "%s/statistics" % (
            self.operon_base_folder)

    def _set_circrna_folder_names(self):
        self.circrna_base_folder = "%s/circRNA" % self.output_folder
        self.circrna_align_folder = "%s/segemehl_align" % (
             self.circrna_base_folder)
        self.circrna_splice_folder = "%s/segemehl_splice" % (
             self.circrna_base_folder)
        self.circrna_circ_folder = "%s/circRNA_tables" % (
             self.circrna_base_folder)
        self.circrna_stat_folder = "%s/statistics" % (
             self.circrna_base_folder)
        self.circrna_gff_folder = "%s/gffs" % (
             self.circrna_base_folder)

    def _set_goterm_folder_names(self):
        self.goterm_base_folder = "%s/GO_term" % self.output_folder
        self.goterm_all_folder = "%s/all_CDS" % self.goterm_base_folder
        self.goterm_express_folder = "%s/expressed_CDS" % (
             self.goterm_base_folder)
        self.goterm_express_result_folder = "%s/GO_term_results" % (
             self.goterm_express_folder)
        self.goterm_express_stat_folder = "%s/statistics" % (
             self.goterm_express_folder)
        self.goterm_all_result_folder = "%s/GO_term_results" % (
             self.goterm_all_folder)
        self.goterm_all_stat_folder = "%s/statistics" % (
             self.goterm_all_folder)

    def _set_starget_folder_names(self):
        self.starget_base_folder = "%s/sRNA_targets" % self.output_folder
        self.starget_RNAplex_folder = "%s/RNAplex" % (
             self.starget_base_folder)
        self.starget_RNAup_folder = "%s/RNAup" % (
             self.starget_base_folder)
        self.starget_merge_folder = "%s/merge" % (
             self.starget_base_folder)
        self.starget_srna_seq_folder = "%s/sRNA_seqs" % (
             self.starget_base_folder)
        self.starget_target_seq_folder = "%s/target_seqs" % (
             self.starget_base_folder)

    def _set_snp_folder_names(self):
        self.snp_base_folder = "%s/SNP_calling" % self.output_folder
        self.ref_snp_folder = "%s/compare_reference" % self.snp_base_folder
        self.tar_snp_folder = "%s/validate_target" % self.snp_base_folder
        self.snp_ref_stat_folder = "%s/statistics" % (
             self.ref_snp_folder)
        self.snp_tar_stat_folder = "%s/statistics" % (
             self.tar_snp_folder)
        self.snp_ref_table_folder = "%s/SNP_table" % (
             self.ref_snp_folder)
        self.snp_tar_table_folder = "%s/SNP_table" % (
             self.tar_snp_folder)
        self.snp_ref_raw_folder = "%s/SNP_raw_outputs" % (
             self.ref_snp_folder)
        self.snp_tar_raw_folder = "%s/SNP_raw_outputs" % (
             self.tar_snp_folder)
        self.snp_ref_seq_folder = "%s/seqs" % (
             self.ref_snp_folder)
        self.snp_tar_seq_folder = "%s/seqs" % (
             self.tar_snp_folder)
        self.snp_ref_seq_extend_BAQ_folder = "%s/extend_BAQ" % (
             self.snp_ref_seq_folder)
        self.snp_tar_seq_extend_BAQ_folder = "%s/extend_BAQ" % (
             self.snp_tar_seq_folder)
        self.snp_ref_seq_with_BAQ_folder = "%s/with_BAQ" % (
             self.snp_ref_seq_folder)
        self.snp_tar_seq_with_BAQ_folder = "%s/with_BAQ" % (
             self.snp_tar_seq_folder)
        self.snp_ref_seq_without_BAQ_folder = "%s/without_BAQ" % (
             self.snp_ref_seq_folder)
        self.snp_tar_seq_without_BAQ_folder = "%s/without_BAQ" % (
             self.snp_tar_seq_folder)

    def _set_ppi_folder_names(self):
        self.ppi_base_folder = "%s/PPI" % self.output_folder
        self.ppi_all_folder = "%s/all_results" % (
             self.ppi_base_folder)
        self.ppi_best_folder = "%s/best_results" % (
             self.ppi_base_folder)
        self.ppi_fig_folder = "%s/figures" % (
             self.ppi_base_folder)

    def _set_sublocal_folder_names(self):
        self.sublocal_base_folder = "%s/subcellular_localization" % (
             self.output_folder)
        self.sublocal_all_folder = "%s/all_CDS" % self.sublocal_base_folder
        self.sublocal_express_folder = "%s/expressed_CDS" % (
             self.sublocal_base_folder)
        self.sublocal_all_results_folder = "%s/psortb_results" % (
             self.sublocal_all_folder)
        self.sublocal_all_stat_folder = "%s/statistics" % (
             self.sublocal_all_folder)
        self.sublocal_express_results_folder = "%s/psortb_results" % (
             self.sublocal_express_folder)
        self.sublocal_express_stat_folder = "%s/statistics" % (
             self.sublocal_express_folder)

    def _set_ribos_folder_names(self):
        self.ribos_base_folder = "%s/riboswitch" % self.output_folder
        self.ribos_gff_folder = "%s/gffs" % (
             self.ribos_base_folder)
        self.ribos_stat_folder = "%s/statistics" % (
             self.ribos_base_folder)
        self.ribos_table_folder = "%s/tables" % (
             self.ribos_base_folder)
        self.ribos_rfam_folder = "%s/scan_Rfam" % (
             self.ribos_base_folder)

    def _set_thermo_folder_names(self):
        self.thermo_base_folder = "%s/RNA_thermometer" % self.output_folder
        self.thermo_gff_folder = "%s/gffs" % (
             self.thermo_base_folder)
        self.thermo_stat_folder = "%s/statistics" % (
             self.thermo_base_folder)
        self.thermo_table_folder = "%s/tables" % (
             self.thermo_base_folder)
        self.thermo_rfam_folder = "%s/scan_Rfam" % (
             self.thermo_base_folder)

    def _set_crispr_folder_names(self):
        self.crispr_base_folder = "%s/crispr" % self.output_folder
        self.crispr_gff_folder = "%s/gffs" % (
             self.crispr_base_folder)
        self.crispr_stat_folder = "%s/statistics" % (
             self.crispr_base_folder)
        self.crispr_data_folder = "%s/CRT_output" % (
             self.crispr_base_folder)

    def required_folders(self, folder_type):
        if (folder_type == "root"):
            return (self.required_base_folders() +
                    self.required_input_folders() +
                    self.required_reference_input_folders() +
                    self.required_wig_folders() +
                    self.required_bam_folders())
        else:
            return (self.required_base_folders() +
                    self.required_input_folders() +
                    self.required_reference_input_folders() +
                    self.required_wig_folders() +
                    self.required_bam_folders() +
                    self.required_output_folders(folder_type))

    def required_base_folders(self):
        return [self.input_folder, self.output_folder]

    def required_input_folders(self):
        return [self.reference_input_folder, self.wig_folder,
                self.mutation_table_folder, self.read_folder,
                self.bam_folder,
                self.database_folder, self.manual_TSS_folder,
                self.manual_pro_folder, self.riborfam_folder,
                self.thermorfam_folder]

    def required_output_folders(self, folder_type):
        folder_dict = {"get_target_fasta": (
                           [self.target_folder] +
                           self.required_target_folders()),
                       "annotation_transfer": (
                           [self.ratt_folder] +
                           self.required_target_folders()),
                       "TSS": (
                           [self.tsspredator_folder] +
                           self.required_tsspredator_folders()),
                       "processing": (
                           [self.processing_site_folder] +
                           self.required_processing_folders()),
                       "terminator": (
                           [self.transterm_folder] +
                           self.required_transterm_folders()),
                       "transcript": (
                           [self.transcript_output_folder] +
                           self.required_transcript_folders()),
                       "utr": [self.utr_folder] + self.required_utr_folders(),
                       "srna": (
                           [self.srna_folder] + self.required_srna_folders()),
                       "sorf": (
                           [self.sorf_folder] + self.required_sorf_folders()),
                       "promoter": [self.promoter_output_folder],
                       "circrna": (
                           [self.circrna_output_folder] +
                           self.required_circrna_folders()),
                       "go_term": (
                           [self.goterm_output_folder] +
                           self.required_goterm_folders()),
                       "srna_target": (
                           [self.starget_output_folder] +
                           self.required_starget_folders()),
                       "snp": (
                           [self.snp_output_folder] +
                           self.required_snp_folders()),
                       "ppi_network": (
                           [self.ppi_output_folder] +
                           self.required_ppi_folders()),
                       "subcellular_localization": (
                           [self.sublocal_output_folder] +
                           self.required_sublocal_folders()),
                       "riboswitch": (
                           [self.ribos_output_folder] +
                           self.required_ribos_folders()),
                       "thermometer": (
                           [self.thermo_output_folder] +
                           self.required_thermo_folders()),
                       "crispr": (
                           [self.crispr_output_folder] +
                           self.required_crispr_folders()),
                       "operon": (
                           [self.operon_output_folder] +
                           self.required_operon_folders())}
        return folder_dict[folder_type]

    def required_reference_input_folders(self):
        return [self.ref_annotation_folder, self.ref_fasta_folder]

    def required_wig_folders(self):
        return [self.tex_folder, self.frag_folder]

    def required_bam_folders(self):
        return [self.bam_ref_folder, self.bam_tar_folder,
                self.bam_ref_tex_folder, self.bam_tar_tex_folder,
                self.bam_ref_frag_folder, self.bam_tar_frag_folder]

    def required_target_folders(self):
        return [self.tar_annotation_folder, self.tar_fasta_folder]

    def required_tsspredator_folders(self):
        return [self.tss_to_gff_folder, self.tss_statistics_folder,
                self.tss_Master_folder, self.tss_config_folder]

    def required_transterm_folders(self):
        return [self.term_to_gff_folder, self.term_statistics_folder,
                self.transtermhp_folder, self.term_to_table_folder]

    def required_processing_folders(self):
        return [self.processing_to_gff_folder,
                self.processing_statistics_folder,
                self.processing_Master_folder,
                self.processing_config_folder]

    def required_transcript_folders(self):
        return [self.transcript_gff_folder, self.transcript_stat_folder,
                self.transcript_table_folder]

    def required_utr_folders(self):
        return [self.utr5_folder, self.utr3_folder,
                self.utr5_stat_folder, self.utr5_gff_folder,
                self.utr3_stat_folder, self.utr3_gff_folder]

    def required_srna_folders(self):
        return [self.srna_gff_folder, self.srna_sec_folder,
                self.srna_sec_plot_folder, self.srna_dot_plot_folder,
                self.srna_mountain_folder, self.srna_table_folder,
                self.srna_blast_folder, self.srna_stat_folder,
                self.srna_gff_class_folder, self.srna_gff_best_folder,
                self.srna_gff_all_folder, self.srna_table_class_folder,
                self.srna_table_best_folder, self.srna_table_all_folder]

    def required_sorf_folders(self):
        return [self.sorf_gff_folder, self.sorf_table_folder,
                self.sorf_stat_folder, self.sorf_gff_best_folder,
                self.sorf_gff_all_folder, self.sorf_table_best_folder,
                self.sorf_table_all_folder]

    def required_operon_folders(self):
        return [self.operon_gff_folder, self.operon_table_folder,
                self.operon_statistics_folder]

    def required_circrna_folders(self):
        return [self.circrna_align_folder, self.circrna_splice_folder,
                self.circrna_circ_folder, self.circrna_stat_folder,
                self.circrna_gff_folder]

    def required_goterm_folders(self):
        return [self.goterm_all_folder, self.goterm_express_folder,
                self.goterm_express_result_folder,
                self.goterm_express_stat_folder,
                self.goterm_all_result_folder, self.goterm_all_stat_folder]

    def required_starget_folders(self):
        return [self.starget_RNAplex_folder, self.starget_RNAup_folder,
                self.starget_merge_folder, self.starget_srna_seq_folder,
                self.starget_target_seq_folder]

    def required_snp_folders(self):
        return [self.ref_snp_folder, self.tar_snp_folder,
                self.snp_ref_stat_folder, self.snp_tar_stat_folder,
                self.snp_ref_table_folder, self.snp_tar_table_folder,
                self.snp_ref_raw_folder, self.snp_tar_raw_folder,
                self.snp_ref_seq_folder, self.snp_tar_seq_folder,
                self.snp_ref_seq_extend_BAQ_folder,
                self.snp_tar_seq_extend_BAQ_folder,
                self.snp_ref_seq_with_BAQ_folder,
                self.snp_tar_seq_with_BAQ_folder,
                self.snp_ref_seq_without_BAQ_folder,
                self.snp_tar_seq_without_BAQ_folder]

    def required_ppi_folders(self):
        return [self.ppi_all_folder, self.ppi_best_folder,
                self.ppi_fig_folder]

    def required_sublocal_folders(self):
        return [self.sublocal_all_folder, self.sublocal_express_folder,
                self.sublocal_all_results_folder,
                self.sublocal_all_stat_folder,
                self.sublocal_express_results_folder,
                self.sublocal_express_stat_folder]

    def required_ribos_folders(self):
        return [self.ribos_gff_folder, self.ribos_table_folder,
                self.ribos_stat_folder, self.ribos_rfam_folder]

    def required_thermo_folders(self):
        return [self.thermo_gff_folder, self.thermo_table_folder,
                self.thermo_stat_folder, self.thermo_rfam_folder]

    def required_crispr_folders(self):
        return [self.crispr_gff_folder, self.crispr_stat_folder,
                self.crispr_data_folder]



from annogesiclib.gff3 import Gff3Parser
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt


def ellipse(x, y, angle, face, al, plt):
    ellipse_ = mpl.patches.Ellipse(xy=(x, y), width=0.8, height=0.3,
                                   angle=angle, facecolor=face, alpha=al)
    plt.gca().add_artist(ellipse_)
    return plt


def line(x, y, angle, plt):
    line_ = mpl.patches.Ellipse(xy=(x, y), width=0.8, height=0.3,
                                angle=angle, facecolor="none",
                                edgecolor="#000000", linewidth=3)
    plt.gca().add_artist(line_)
    return plt


def plot_text(plt, xy1, xy2, tss_type, size, color_text):
    plt.text(xy1, xy2, tss_type, ha="center",
             va="center", fontsize=15, fontweight='bold',
             color=color_text)


def text_total(xy, tss_type, num, plt):
    plot_text(plt, xy[0], xy[1], tss_type, 15, "black")
    if tss_type != "Orphan":
        plot_text(plt, xy[0], xy[1] - 0.05, str(num), 15, "black")


def text(xy, tss_type, num, plt):
    if (tss_type == "Primary") or (
            tss_type == "Antisense") or (
            tss_type == "Antisense_Primary"):
        plot_text(plt, xy[0], xy[1], str(num), 16, "white")
    else:
        plot_text(plt, xy[0], xy[1], str(num), 16, "black")


def check_tss_class(total_types, strain, tss, tss_type):
    if tss_type not in total_types[strain].keys():
        total_types[strain][tss_type] = 0
    if tss_type in tss.attributes["type"]:
        total_types[strain][tss_type] += 1


def import_types(tsss):
    types = {"all": {}}
    total_types = {"all": {}}
    for strain, datas in tsss.items():
        if strain not in types.keys():
            types[strain] = {}
            total_types[strain] = {}
        for tss in datas:
            check_tss_class(total_types, strain, tss, "Primary")
            check_tss_class(total_types, strain, tss, "Secondary")
            check_tss_class(total_types, strain, tss, "Internal")
            check_tss_class(total_types, strain, tss, "Antisense")
            check_tss_class(total_types, strain, tss, "Orphan")
            sorted_types = sorted(tss.attributes["type"].split("&"))
            ty = None
            for tss_type in sorted_types:
                if ty is None:
                    ty = tss_type
                else:
                    if tss_type not in ty:
                        ty = ty + "_" + tss_type
            if ty not in types[strain].keys():
                types[strain][ty] = 0
            types[strain][ty] += 1
    return types, total_types


def read_gff(tss_file):
    tsss = {"all": []}
    tss_num = {"all": 0}
    pre_strain = ""
    gff_parser = Gff3Parser()
    f_h = open(tss_file)
    for entry in gff_parser.entries(f_h):
        if pre_strain != entry.seq_id:
            tsss[entry.seq_id] = []
            tss_num[entry.seq_id] = 0
            pre_strain = entry.seq_id
        tsss[entry.seq_id].append(entry)
        tsss["all"].append(entry)
        tss_num[entry.seq_id] += 1
        tss_num["all"] += 1
    for strain in tsss.keys():
        tsss[strain] = sorted(tsss[strain], key=lambda k: (k.seq_id, k.start))
    f_h.close()
    return tsss, tss_num


def plot(types, file_type, feature_name, total_types, tss_num):
    for strain, tss_types in types.items():
        if len(types.keys()) <= 2:
            if strain == "all":
                continue
        plt.figure(figsize=(12, 6))
        coordinate_total = {"Primary": (0.05, 0.85), "Secondary": (0.2, 0.95),
                            "Internal": (0.575, 0.95),
                            "Antisense": (0.7, 0.85), "Orphan": (0.8, 0.3)}
        if feature_name == "processing site":
            plot_text(plt, 0.05, 0.05, "Total processing sites", 15, "black")
            plot_text(plt, 0.05, 0, str(tss_num[strain]), 15, "black")
        elif feature_name == "TSS":
            plot_text(plt, 0.025, 0.05, "Total TSSs", 15, "black")
            plot_text(plt, 0.025, 0, str(tss_num[strain]), 15, "black")
        for tss_type, num in total_types[strain].items():
            text_total(coordinate_total[tss_type], tss_type, num, plt)
        ellipse(0.5, 0.4, 70, "#E83241", 1.0, plt)
        ellipse(0.25, 0.4, -70, "#6648DC", 0.8, plt)
        ellipse(0.38, 0.495, 70, "#13C139", 0.5, plt)
        ellipse(0.37, 0.495, -70, "#E8D632", 0.4, plt)
        circ = mpl.patches.Ellipse(xy=(0.8, 0.2), width=0.09, height=0.15,
                                   facecolor='none', edgecolor="#000000",
                                   linewidth=3)
        plt.gca().add_artist(circ)
        line(0.25, 0.4, -70, plt)
        line(0.37, 0.495, -70, plt)
        line(0.5, 0.4, 70, plt)
        line(0.38, 0.495, 70, plt)
        coordinates = {"Primary": (0.15, 0.5), "Secondary": (0.275, 0.75),
                       "Internal": (0.476, 0.75), "Antisense": (0.625, 0.5),
                       "Primary_Secondary": (0.225, 0.625),
                       "Internal_Primary": (0.25, 0.225),
                       "Antisense_Primary": (0.375, 0.075),
                       "Internal_Secondary": (0.375, 0.625),
                       "Antisense_Secondary": (0.5, 0.225),
                       "Antisense_Internal": (0.525, 0.625),
                       "Internal_Primary_Secondary": (0.3, 0.45),
                       "Antisense_Primary_Secondary": (0.42, 0.18),
                       "Antisense_Internal_Primary": (0.335, 0.18),
                       "Antisense_Internal_Secondary": (0.45, 0.45),
                       "Antisense_Internal_Primary_Secondary": (0.375, 0.3),
                       "Orphan": (0.8, 0.19)}
        for tss_type, xy in coordinates.items():
            if tss_type not in tss_types.keys():
                tss_types[tss_type] = 0
            text(xy, tss_type, tss_types[tss_type], plt)
        plt.axis('off')
        plt.savefig("_".join([file_type, "venn", strain + ".png"]))
        plt.clf()


def plot_venn(tss_file, file_type):
    if file_type == "processing":
        feature_name = "processing site"
    else:
        feature_name = "TSS"
    types = {"all": {}}
    total_types = {"all": {}}
    tsss, tss_num = read_gff(tss_file)
    types, total_types = import_types(tsss)
    plot(types, file_type, feature_name, total_types, tss_num)



import os
import csv
import shutil
from annogesiclib.gff3 import Gff3Parser


def filter_frag(srna_table, srna_gff):
    out = open("tmp_srna.gff", "w")
    out_ta = open("tmp_srna.csv", "w")
    out.write("##gff-version 3\n")
    gffs = []
    tables = []
    gff_parser = Gff3Parser()
    g_f = open(srna_gff, "r")
    for entry in gff_parser.entries(g_f):
        gffs.append(entry)
    fh = open(srna_table, "r")
    for row in csv.reader(fh, delimiter='\t'):
        tables.append(row)
    new_gffs = []
    for gff in gffs:
        if ("UTR_type" in gff.attributes.keys()):
            if (gff.attributes["UTR_type"] == "5utr") or (
                    gff.attributes["UTR_type"] == "interCDS"):
                for table in tables:
                    if (gff.seq_id == table[0]) and (
                            gff.start == int(table[2])) and (
                            gff.end == int(table[3])) and (
                            gff.strand == table[4]):
                        if "frag" in table[5]:
                            new_gffs.append(gff)
            elif gff.attributes["UTR_type"] == "3utr":
                new_gffs.append(gff)
        else:
            new_gffs.append(gff)
    new_tables = []
    for table in tables:
        for gff in new_gffs:
            if (gff.seq_id == table[0]) and (
                 gff.start == int(table[2])) and (
                 gff.end == int(table[3])) and (
                 gff.strand == table[4]):
                new_tables.append(table)
                out_ta.write("\t".join(table) + "\n")
    for gff in new_gffs:
        for table in new_tables:
            if (gff.seq_id == table[0]) and (
                 gff.start == int(table[2])) and (
                 gff.end == int(table[3])) and (
                 gff.strand == table[4]):
                out.write(gff.info + "\n")
    g_f.close()
    fh.close()
    os.remove(srna_gff)
    os.remove(srna_table)
    shutil.move("tmp_srna.gff", srna_gff)
    shutil.move("tmp_srna.csv", srna_table)



import os
import sys
import csv
import shutil
from glob import glob
from subprocess import call
from annogesiclib.multiparser import Multiparser
from annogesiclib.seq_editer import SeqEditer
from annogesiclib.transcript_SNP import snp_detect
from annogesiclib.helper import Helper


class SNPCalling(object):
    '''detection of SNP'''

    def __init__(self, args_snp):
        self.multiparser = Multiparser()
        self.seq_editer = SeqEditer()
        self.helper = Helper()
        if args_snp.types == "reference":
            file_type = "compare_reference"
        else:
            file_type = "validate_target"
        self.seq_path = os.path.join(args_snp.out_folder, file_type, "seqs")
        self.stat_path = os.path.join(args_snp.out_folder, file_type,
                                      "statistics")
        self.fasta_path = os.path.join(args_snp.fastas, "tmp")
        self.outputs = {"table": os.path.join(
                        args_snp.out_folder, file_type, "SNP_table"),
                        "raw": os.path.join(
                        args_snp.out_folder, file_type, "SNP_raw_outputs"),
                        "tmp": os.path.join(args_snp.out_folder, "tmp_bcf"),
                        "depth": os.path.join(args_snp.out_folder, "tmp_depth")}
        if "whole_reads.bam" in os.listdir(args_snp.out_folder):
            self.helper.remove_all_content(args_snp.out_folder,
                                           "whole_read", "file")
        self.bams = {"whole": os.path.join(args_snp.out_folder,
                                           "whole_reads.bam"),
                     "sort": os.path.join(args_snp.out_folder,
                                          "whole_reads_sorted.bam"),
                     "bams": []}
        self.header = os.path.join(args_snp.out_folder, "header")
        self.baqs = {"with": "with_BAQ", "without": "without_BAQ",
                     "extend": "extend_BAQ"}

    def _transcript_snp(self, fasta, snp, out_table_prefix, type_,
                        prefix, bam_number, table_path, args_snp):
        seq_path = os.path.join(self.seq_path, self.baqs[type_], prefix)
        stat_prefix = os.path.join(self.stat_path, "_".join([
            "stat", "_".join([prefix, self.baqs[type_]]), "SNP"]))
        snp_detect(fasta, snp, self.outputs["depth"], out_table_prefix,
                   os.path.join(seq_path, prefix), bam_number,
                   stat_prefix, args_snp)
        self.helper.move_all_content(table_path, self.stat_path, [".png"])

    def _get_para(self, args_snp):
        bams = self.bams["sort"]
        if args_snp.caller == "c":
            bcf_para = "-vcO"
        else:
            bcf_para = "-vmO"
        return bams, bcf_para

    def _run_tools(self, fasta_file, out_raw_prefix, type_, args_snp):
        bams, bcf_para = self._get_para(args_snp)
        if type_ == "with":
            command = [args_snp.samtools_path, "mpileup", "-t", "DP"]
        elif type_ == "without":
            command = [args_snp.samtools_path, "mpileup", "-t", "DP", "-B"]
        elif type_ == "extend":
            command = [args_snp.samtools_path, "mpileup", "-t", "DP", "-E"]
        if args_snp.rg:
            command = command + ["-ugf", fasta_file, bams]
        else:
            command = command + ["--ignore-RG", "-ugf", fasta_file, bams]
        os.system(" ".join(command) + ">" + self.outputs["tmp"])
        out_vcf = "_".join([out_raw_prefix, self.baqs[type_] + ".vcf"])
        if args_snp.chrom == "1":
            call([args_snp.bcftools_path, "call", "--ploidy", args_snp.chrom,
                  self.outputs["tmp"], bcf_para, "v", "-o", out_vcf])
        elif args_snp.chrom == "2":
            call([args_snp.bcftools_path, "call",
                  self.outputs["tmp"], bcf_para, "v", "-o", out_vcf])
        return out_vcf

    def _run_sub(self, args_snp, fasta_file, type_, file_prefixs, prefix,
                 table_path, bam_number):
        out_vcf = self._run_tools(fasta_file, file_prefixs["raw_prefix"],
                                  type_, args_snp)
        self.helper.check_make_folder(
             os.path.join(self.seq_path, self.baqs[type_], prefix))
        self._transcript_snp(
            fasta_file, out_vcf,
            "_".join([file_prefixs["table_prefix"], self.baqs[type_]]),
            type_, prefix, bam_number, table_path, args_snp)

    def _run_program(self, fasta_file, file_prefixs, prefix, bam_number,
                     table_path, args_snp):
        for index in args_snp.program:
            if index == "with_BAQ":
                type_ = "with"
                print("Running SNP calling with BAQ")
            elif index == "without_BAQ":
                type_ = "without"
                print("Running SNP calling without BAQ")
            elif index == "extend_BAQ":
                print("Running SNP calling extend BAQ")
                type_ = "extend"
            else:
                print("Error: No correct program, please assign "
                      "\"with_BAQ\", \"without_BAQ\", \"extend_BAQ\"!")
                sys.exit()
            self._run_sub(args_snp, fasta_file, type_, file_prefixs, prefix,
                          table_path, bam_number)

    def _detect_fasta(self, fasta):
        detect = False
        if fasta.endswith(".fa"):
            prefix = fasta[:-3]
            detect = True
        elif fasta.endswith(".fna"):
            prefix = fasta[:-4]
            detect = True
        elif fasta.endswith(".fasta"):
            prefix = fasta[:-6]
            detect = True
        return (detect, prefix)

    def _run_bam(self, samtools_path, sub_command, bam_file):
        if sub_command == "merge":
            command = (" ".join([samtools_path, sub_command,
                       self.bams["whole"], bam_file]))
        elif sub_command == "sort":
            command = (" ".join([samtools_path, sub_command,
                                 "-o", bam_file, self.bams["whole"]]))
        os.system(command)
        self.bams["bams"].append(bam_file.replace(".bam", "_sort.bam"))

    def _merge_bams(self, args_snp):
        bams = []
        num_normal = 0
        num_frag = 0
        if (args_snp.bams is None):
            print("Error: There is no BAMs folders!!")
            sys.exit()
        else:
            num_bam = 0
            for files in args_snp.bams:
                for bam in glob(files):
                    bams.append(bam)
                    num_bam += 1
        if num_bam <= 1:
            shutil.copyfile(bams[0], self.bams["whole"])
            print("Sorting BAM file now")
            self._run_bam(args_snp.samtools_path, "sort",
                          self.bams["sort"])
        else:
            print("Merging BAM files now")
            self._run_bam(args_snp.samtools_path, "merge",
                          " ".join(bams))
            print("Sorting BAM file now")
            self._run_bam(args_snp.samtools_path, "sort",
                          self.bams["sort"])
        out_depth = open(self.outputs["depth"], "w")
        call([args_snp.samtools_path, "index",  self.bams["sort"]])
        call([args_snp.samtools_path, "depth",  self.bams["sort"]],
             stdout=out_depth)
        return num_bam

    def _modify_header(self, fastas):
        for fasta in os.listdir(fastas):
            if fasta.endswith("fasta") or \
               fasta.endswith("fa") or \
               fasta.endswith("fna"):
                self.seq_editer.modify_header(os.path.join(fastas, fasta))

    def _get_header(self, samtools_path, bam, seq_names):
        command = " ".join([samtools_path, "view", "-H", bam])
        os.system(">".join([command, self.header]))
        fh = open(self.header, "r")
        for row in csv.reader(fh, delimiter="\t"):
            if row[0] == "@SQ":
                seq_names.append(row[1].split(":")[1])
        fh.close()

    def _get_genome_name(self, args_snp):
        seq_names = []
        self._get_header(args_snp.samtools_path, self.bams["sort"],
                         seq_names)
        return seq_names

    def _remove_bams(self):
        if os.path.exists(self.bams["whole"]):
            os.remove(self.bams["whole"])
        if os.path.exists(self.bams["whole"] + ".bai"):
            os.remove(self.bams["whole"] + ".bai")
        if os.path.exists(self.bams["sort"]):
            os.remove(self.bams["sort"])
        if os.path.exists(self.bams["sort"] + ".bai"):
            os.remove(self.bams["sort"] + ".bai")
        if os.path.exists(self.header):
            os.remove(self.header)
        os.remove(self.outputs["depth"])

    def run_snp_calling(self, args_snp):
        self.multiparser.parser_fasta(args_snp.fastas)
        self._modify_header(args_snp.fastas)
        bam_number = self._merge_bams(args_snp)
        seq_names = self._get_genome_name(args_snp)
        if ("with_BAQ" not in args_snp.program) and (
                "without_BAQ" not in args_snp.program) and (
                "extend_BAQ" not in args_snp.program):
            print("Error: Please assign a correct programs: "
                  "\"with_BAQ\", \"without_BAQ\", \"extend_BAQ\".")
            sys.exit()
        else:
            for fasta in os.listdir(self.fasta_path):
                if (fasta.split(".f")[0] in seq_names):
                    fasta_datas = self._detect_fasta(fasta)
                    detect = fasta_datas[0]
                    prefix = fasta_datas[1]
                    if detect:
                        detect = False
                        print("Computing {0} now".format(fasta))
                        self.helper.check_make_folder(
                             os.path.join(self.outputs["table"], prefix))
                        self.helper.check_make_folder(
                             os.path.join(self.outputs["raw"], prefix))
                        file_prefixs = {"raw_prefix": os.path.join(
                                        self.outputs["raw"], prefix, prefix),
                                        "table_prefix": os.path.join(
                                        self.outputs["table"], prefix, prefix)}
                        fasta_file = os.path.join(self.fasta_path, fasta)
                        table_path = os.path.join(self.outputs["table"],
                                                  prefix)
                        self._run_program(fasta_file, file_prefixs, prefix,
                                          bam_number, table_path, args_snp)
                        os.remove(self.outputs["tmp"])
        self.helper.remove_tmp_dir(args_snp.fastas)
        self._remove_bams()



import os
import shutil
from annogesiclib.multiparser import Multiparser
from annogesiclib.seq_editer import SeqEditer
from annogesiclib.helper import Helper


class TargetFasta(object):
    '''detection of sRNA target interaction'''

    def __init__(self, tar_folder, ref_folder):
        self.multiparser = Multiparser()
        self.seq_editer = SeqEditer()
        self.helper = Helper()
        self.folders = {"tmp_tar": os.path.join(tar_folder, "tmp")}

    def gen_folder(self, out_folder, ref_files):
        new_ref_folder = os.path.join(out_folder, "tmp_reference")
        self.helper.check_make_folder(new_ref_folder)
        for file_ in ref_files:
            shutil.copy(file_, new_ref_folder)
        self.folders["tmp_ref"] = os.path.join(new_ref_folder, "tmp")
        self.multiparser.parser_fasta(new_ref_folder)
        if "tmp_tar" in os.listdir(out_folder):
            shutil.rmtree(self.folders["tmp_tar"])
        os.mkdir(self.folders["tmp_tar"])
        return new_ref_folder

    def get_target_fasta(self, mut_table, tar_folder, ref_files,
                         output, out_folder):
        new_ref_folder = self.gen_folder(out_folder, ref_files)
        self.seq_editer.modify_seq(self.folders["tmp_ref"], mut_table,
                                   self.folders["tmp_tar"])
        print("Transfering to target fasta")
        for file_ in output:
            first = True
            datas = file_.split(":")
            filename = datas[0]
            strains = datas[1].split(",")
            out = open(filename, "w")
            for strain in strains:
                if strain + ".fa" in os.listdir(self.folders["tmp_tar"]):
                    if first:
                        first = False
                    else:
                        out.write("\n")
                    with open(os.path.join(
                              self.folders["tmp_tar"],
                              strain + ".fa")) as f_h:
                        for line in f_h:
                            out.write(line)
                else:
                    print("Error: No fasta information of {0}.fa".format(
                          strain))
            out.close()
        shutil.rmtree(self.folders["tmp_tar"])
        shutil.rmtree(self.folders["tmp_ref"])
        if "tmp_reference" in os.listdir(out_folder):
            shutil.rmtree(new_ref_folder)
        print("Please use the new fasta file to remapping again.")



from annogesiclib.gff3 import Gff3Parser


def detect_energy(line, srna):
    duplex = line.split(" ")[0]
    if ("(" in duplex) or (")" in duplex):
        energy = line.split(":")[1].split("(")[1].split("=")[0]
        energy = float(energy.strip())
        if energy < srna["energy"]:
            srna["energy"] = energy


def mod_srna_tar_pos(gff, pos, type_, pre_target, suf_target, length):
    start = int(pos.split(",")[0])
    end = int(pos.split(",")[-1])
    if (gff.strand == "+"):
        if type_ == "srna":
            g_start = gff.start + start - 1
            g_end = gff.start + end - 1
        else:
            if (gff.start - pre_target) <= 0:
                g_start = start
                g_end = end
            else:
                g_start = gff.start - pre_target + start - 1
                g_end = gff.start - pre_target + end - 1
            if (gff.start - pre_target + end - 1) > length:
                g_end = length
    else:
        if type_ == "srna":
            g_end = gff.end - start + 1
            g_start = gff.end - end + 1
        else:
            if (gff.end + pre_target) > length:
                g_start = length - end + 1
                g_end = length - start + 1
            else:
                g_start = gff.end + pre_target - end + 1
                g_end = gff.end + pre_target - start + 1
            if (gff.end + pre_target - end + 1) <= 0:
                g_start = 1
    return g_start, g_end

def print_rank_one(srnas, out, feature, gffs, srna_gffs, args_tar, length):
    out.write("\t".join(["sRNA", "strain", "sRNA_position",
                         "sRNA_interacted_position_" + feature,
                         "sRNA_strand", "target_gene_ID", "target_ID",
                         "target_locus_tag", "target_position",
                         "target_interacted_position_" + feature,
                         "target_strand", "energy_" + feature,
                         "rank_" + feature]) + "\n")
    for method, srna_datas in srnas.items():
        for srna_id, targets in srna_datas.items():
            rank = 0
            sort_targets = sorted(targets, key=lambda k: (k["energy"]))
            for target in sort_targets:
                if target["energy"] < 0:
                    rank += 1
                    target["rank"] = rank
                    if rank <= args_tar.top:
                        if method == feature:
                            srna_infos = get_srna_name(srna_gffs, srna_id)
                            name = srna_infos[0]
                            srna_info = srna_infos[1]
                            target_info = get_target_info(gffs, target)
                            s_start, s_end = mod_srna_tar_pos(
                                    srna_info, target["srna_pos"], "srna",
                                    args_tar.tar_start, args_tar.tar_end,
                                    length)
                            t_start, t_end = mod_srna_tar_pos(
                                    target_info, target["tar_pos"], "tar",
                                    args_tar.tar_start, args_tar.tar_end,
                                    length)
                            if srna_info is not None:
                                out.write("\t".join([
                                    name, str(srna_info.seq_id),
                                    "-".join([str(srna_info.start),
                                              str(srna_info.end)]),
                                    "-".join([str(s_start), str(s_end)]),
                                    srna_info.strand, target["gene_id"],
                                    target["target_id"],
                                    target["target_locus"],
                                    "-".join([str(target_info.start),
                                              str(target_info.end)]),
                                    "-".join([str(t_start), str(t_end)]),
                                    target_info.strand, str(target["energy"]),
                                    str(target["rank"])]) + "\n")

def extract_pos(line, srnas, method):
    if (line.startswith("(")) or (
            line.startswith(")")) or (
            line.startswith(".")):
        tar_pos = line.split(":")[0].strip().split(" ")[-1]
        srna_pos = line.split(":")[-1].strip().split(" ")[0]
        srnas["tar_pos"] = tar_pos
        srnas["srna_pos"] = srna_pos

def read_table(gffs, rnaplex, rnaup, genes, genomes, features):
    srnas = {"RNAup": {}, "RNAplex": {}}
    start = False
    count_seq = 0
    srna_names = set()
    for gff in gffs:
        if gff.attributes["ID"] not in srna_names:
            srna_names.add(gff.attributes["ID"])
    if rnaplex is not None:
        with open(rnaplex, "r") as p_h:
            for line in p_h:
                line = line.strip()
                if line.startswith(">"):
                    start = True
                    count_seq += 1
                    if count_seq == 1:
                        tags = line[1:].split("|")
                        target_locus = tags[0]
                        target_id = tags[1]
                        detail = tags[-1]
                        gene_id = get_gene_id(detail, target_id,
                                              genes, genomes, features)
                    elif count_seq == 2:
                        srna = line[1:]
                        if srna not in srnas["RNAplex"].keys():
                            srnas["RNAplex"][srna] = []
                        srnas["RNAplex"][srna].append({
                            "target_id": target_id, "gene_id": gene_id,
                            "target_locus": target_locus,
                            "detail": detail, "energy": 0})
                        count_seq = 0
                else:
                    if start:
                        detect_energy(line, srnas["RNAplex"][srna][-1])
                        extract_pos(line, srnas["RNAplex"][srna][-1], "RNAplex")
    if rnaup is not None:
        with open(rnaup, "r") as u_h:
            for line in u_h:
                line = line.strip()
                if line.startswith(">"):
                    if line[1:] in srna_names:
                        srna = line[1:]
                    else:
                        tags = line[1:].split("|")
                        gene_id = get_gene_id(tags[-1], tags[1], genes, genomes,
                                              features)
                        if srna in srnas["RNAup"].keys():
                            srnas["RNAup"][srna].append({
                                "target_id": tags[1], "target_locus": tags[0],
                                "detail": tags[-1], "energy": 0,
                                "gene_id": gene_id})
                        else:
                            srnas["RNAup"][srna] = []
                            srnas["RNAup"][srna].append({
                                "target_id": tags[1], "target_locus": tags[0],
                                "detail": tags[-1], "energy": 0,
                                "gene_id": gene_id})
                else:
                    detect_energy(line, srnas["RNAup"][srna][-1])
                    extract_pos(line, srnas["RNAup"][srna][-1], "RNAplex")
    return srnas


def get_gene_id(detail, tar_id, genes, gffs, features):
    for gff in gffs:
        if gff.feature in features:
            if tar_id != "NA":
                if "ID" in gff.attributes.keys():
                    if gff.attributes["ID"] == tar_id:
                        tar = gff
                        break
            else:
                start = int(detail.split("-")[0])
                end = int(detail.split("-")[-1].split("_")[0])
                strand = int(detail.split("_")[-1])
                if (gff.start == start) and (gff.end == end) and (
                        gff.strand == strand):
                    tar = gff
                    break
    gene_id = "NA"
    for gene in genes:
        if "Parent" in tar.attributes.keys():
            if "ID" in gene.attributes.keys():
                if (gene.attributes["ID"] in
                        tar.attributes["Parent"].split(",")):
                    gene_id = gene.attributes["ID"]
                    return gene_id
        else:
            if (gene.seq_id == tar.seq_id) and (
                    gene.strand == tar.strand):
                if ((tar.start <= gene.start) and (
                        tar.end >= gene.end)) or (
                        (tar.start >= gene.start) and (
                        tar.end <= gene.end)) or (
                        (tar.start <= gene.start) and (
                        tar.end <= gene.end) and (
                        tar.end >= gene.start)) or (
                        (tar.start >= gene.start) and (
                        tar.start <= gene.end) and (
                        tar.end >= gene.end)):
                    if "ID" in gene.attributes.keys():
                        gene_id = gene.attributes["ID"]
                        return gene_id
    return gene_id


def import_merge(merges, name, srna_info, srna_plex, srna_up, target_info,
                 pre_target, suf_target, length):
    ps_start, ps_end = mod_srna_tar_pos(
            srna_info, srna_plex["srna_pos"], "srna", pre_target,
            suf_target, length)
    pt_start, pt_end = mod_srna_tar_pos(
            target_info, srna_plex["tar_pos"], "tar", pre_target,
            suf_target, length)
    us_start, us_end = mod_srna_tar_pos(
            srna_info, srna_up["srna_pos"], "srna", pre_target,
            suf_target, length)
    ut_start, ut_end = mod_srna_tar_pos(
            target_info, srna_up["tar_pos"], "tar", pre_target,
            suf_target, length)
    merges.append([name, srna_info.seq_id,
                   "-".join([str(srna_info.start), str(srna_info.end)]),
                   "-".join([str(ps_start), str(ps_end)]),
                   "-".join([str(us_start), str(us_end)]),
                   srna_info.strand, srna_plex["gene_id"],
                   srna_plex["target_id"], srna_plex["target_locus"],
                   "-".join([str(target_info.start), str(target_info.end)]),
                   "-".join([str(pt_start), str(pt_end)]),
                   "-".join([str(ut_start), str(ut_end)]),
                   target_info.strand,
                   str(srna_plex["energy"]), str(srna_plex["rank"]),
                   str(srna_up["energy"]), str(srna_up["rank"])])


def get_srna_name(gffs, srna):
    detect_name = False
    srna_info = None
    for gff in gffs:
        if (gff.attributes["ID"] == srna) and \
           ("Name" in gff.attributes.keys()):
            name = gff.attributes["Name"]
            detect_name = True
            srna_info = gff
            break
        elif (gff.attributes["ID"] == srna):
            srna_info = gff
            break
    if not detect_name:
        name = srna
    return (name, srna_info)


def get_target_info(gffs, target):
    for gff in gffs:
        if (str(gff.start) + "-" +
            str(gff.end) + "_" + gff.strand) == target["detail"]:
            target_info = gff
            if ("locus_tag" in gff.attributes.keys()) and (
                    "gene" in gff.attributes.keys()):
                if (gff.attributes["gene"] not in target["target_locus"]):
                    target["target_locus"] = "|".join([target["target_locus"],
                                                       gff.attributes["gene"]])
            elif ("locus_tag" in gff.attributes.keys()) and (
                    "Name" in gff.attributes.keys()):
                if (gff.attributes["Name"] not in target["target_locus"]):
                    target["target_locus"] = "|".join([target["target_locus"],
                                                       gff.attributes["Name"]])
            return target_info


def print_file(merges, out):
    merges = sorted(merges, key=lambda k: (k[0], int(k[14])))
    for merge in merges:
        out.write("\t".join(merge) + "\n")


def read_gff(filename):
    gffs = []
    genes = []
    for entry in Gff3Parser().entries(open(filename)):
        if entry.feature == "gene":
            genes.append(entry)
        gffs.append(entry)
    gffs = sorted(gffs, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    if len(genes) != 0:
        genes = sorted(genes, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    return gffs, genes


def print_title(out):
    out.write("\t".join(["sRNA", "strain", "sRNA_position",
                         "sRNA_interacted_position_RNAplex",
                         "sRNA_interacted_position_RNAup", "sRNA_strand",
                         "target_gene_ID", "target_ID", "target_locus_tag", "target_position",
                         "target_interacted_position_RNAplex",
                         "target_interacted_position_RNAup", "target_strand",
                         "energy_RNAplex", "rank_RNAplex",
                         "energy_RNAup", "rank_RNAup"]) + "\n")


def merge_base_rnaplex(srnas, srna_gffs, args_tar, gffs, merges, length):
    '''merge the results based on the ranking of RNAplex'''
    overlaps = []
    for srna, srna_plexs in srnas["RNAplex"].items():
        srna_datas = get_srna_name(srna_gffs, srna)
        name = srna_datas[0]
        srna_info = srna_datas[1]
        for srna_plex in srna_plexs:
            if "rank" in srna_plex.keys():
                if ("print" not in srna_plex.keys()) and (
                        srna_plex["rank"] <= args_tar.top):
                    for srna_up in srnas["RNAup"][srna]:
                        if (srna_plex["detail"] == srna_up["detail"]) and (
                                "rank" in srna_up.keys()):
                            if (srna_up["rank"] <= args_tar.top) and (
                                    "print" not in srna_up.keys()):
                                target_info = get_target_info(
                                    gffs, srna_plex)
                                import_merge(
                                    overlaps, name, srna_info, srna_plex,
                                    srna_up, target_info, args_tar.tar_start,
                                    args_tar.tar_end, length)
                                srna_plex["print"] = True
                                srna_up["print"] = True
                                import_merge(
                                    merges, name, srna_info, srna_plex,
                                    srna_up, target_info, args_tar.tar_start,
                                    args_tar.tar_end, length)
                            elif ("print" not in srna_up.keys()):
                                target_info = get_target_info(
                                    gffs, srna_plex)
                                import_merge(
                                    merges, name, srna_info, srna_plex,
                                    srna_up, target_info, args_tar.tar_start,
                                    args_tar.tar_end, length)
                                srna_plex["print"] = True
                        elif (srna_plex["detail"] == srna_up["detail"]) and (
                                "rank" not in srna_up.keys()):
                            srna_up["rank"] = "NA"
                            srna_up["energy"] = "NA"
                            if ("print" not in srna_up.keys()):
                                target_info = get_target_info(
                                    gffs, srna_plex)
                                import_merge(
                                    merges, name, srna_info, srna_plex,
                                    srna_up, target_info, args_tar.tar_start,
                                    args_tar.tar_end, length)
                                srna_plex["print"] = True
    return overlaps


def merge_base_rnaup(srnas, srna_gffs, args_tar, gffs, merges, length):
    '''merge the results based on the ranking of RNAup'''
    for srna, srna_ups in srnas["RNAup"].items():
        srna_datas = get_srna_name(srna_gffs, srna)
        name = srna_datas[0]
        srna_info = srna_datas[1]
        for srna_up in srna_ups:
            if "rank" in srna_up.keys():
                if srna_up["rank"] != "NA":
                    if ("print" not in srna_up.keys()) and (
                            srna_up["rank"] <= args_tar.top) and (
                            srna in srnas["RNAplex"].keys()):
                        for srna_plex in srnas["RNAplex"][srna]:
                            if (srna_plex["detail"] == srna_up["detail"]):
                                if ("rank" in srna_plex.keys()) and (
                                        "print" not in srna_plex.keys()):
                                    target_info = get_target_info(
                                        gffs, srna_up)
                                    import_merge(
                                      merges, name, srna_info, srna_plex,
                                      srna_up, target_info, args_tar.tar_start,
                                      args_tar.tar_end, length)
                                    srna_up["print"] = True
                                elif ("rank" not in srna_plex.keys()) and (
                                        "print" not in srna_plex.keys()):
                                    srna_plex["rank"] = "NA"
                                    srna_plex["energy"] = "NA"
                                    import_merge(
                                      merges, name, srna_info, srna_plex,
                                      srna_up, target_info, args_tar.tar_start,
                                      args_tar.tar_end, length)
                                    srna_up["print"] = True


def read_fasta(seq_file):
    length = 0
    with open(seq_file) as fh:
        for line in fh:
            line = line.strip()
            if not line.startswith(">"):
                length = length + len(line)
    return length


def merge_srna_target(rnaplex, rnaup, args_tar, out_rnaplex, out_rnaup,
                      seq_file, output, out_overlap, srna_gff_file,
                      annotation_gff):
    '''merge the results of RNAup and RNAplex'''
    length = read_fasta(seq_file)
    merges = []
    srna_gffs, NA = read_gff(srna_gff_file)
    gffs, genes = read_gff(annotation_gff)
    srnas = read_table(srna_gffs, rnaplex, rnaup, genes, gffs,
                       args_tar.features)
    if out_rnaplex is not None:
        out_p = open(out_rnaplex, "w")
        print_rank_one(srnas, out_p, "RNAplex", gffs, srna_gffs, args_tar,
                       length)
    if out_rnaup is not None:
        out_u = open(out_rnaup, "w")
        print_rank_one(srnas, out_u, "RNAup", gffs, srna_gffs, args_tar,
                       length)
    if (out_rnaup is not None) and (out_rnaplex is not None):
        out_m = open(output, "w")
        out_o = open(out_overlap, "w")
        print_title(out_m)
        print_title(out_o)
        print("Merging now...")
        overlaps = merge_base_rnaplex(srnas, srna_gffs, args_tar, gffs,
                                      merges, length)
        merge_base_rnaup(srnas, srna_gffs, args_tar, gffs, merges, length)
        print_file(merges, out_m)
        print_file(overlaps, out_o)



import os
import shutil
import math
from annogesiclib.gff3 import Gff3Parser


def read_gff(input_file):
    datas = []
    gff_parser = Gff3Parser()
    f_h = open(input_file, "r")
    for entry in gff_parser.entries(f_h):
        datas.append(entry)
    datas = sorted(datas, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    return datas


def compare_tss_pro(tars, refs, out, cluster):
    '''compare between TSS and processing site'''
    for tar in tars:
        for ref in refs:
            if (tar.seq_id == ref.seq_id) and (
                    tar.strand == ref.strand):
                if math.fabs(tar.start - ref.start) <= cluster:
                    break
                elif (ref.start - tar.start) > cluster:
                    out.write(tar.info + "\n")
                    break


def filter_tss_pro(tss_file, pro_file, feature, cluster):
    '''deal with the overlap of TSS and processing site'''
    tsss = read_gff(tss_file)
    pros = read_gff(pro_file)
    out = open("tmp_filter", "w")
    out.write("##gff-version 3\n")
    if feature.lower() == "tss":
        compare_tss_pro(pros, tsss, out, cluster)
        os.remove(pro_file)
        shutil.move("tmp_filter", pro_file)
    elif feature.lower() == "processing_site":
        compare_tss_pro(tsss, pros, out, cluster)
        os.remove(tss_file)
        shutil.move("tmp_filter", tss_file)
